/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.common;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.TreeMap;

/**
 * PropertyManager.java
 * 
 * This class will contain all the Property needed during parsing and all other
 * action This class is Design to be used as a Singloton.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">Sbastien Bianco</a>
 * 
 * Creation Date : Dec 6, 2006 2:51:04 PM
 */
public class PropertyManager {

	/* ********************** */
	/* * Parsing properties * */
	/* ********************** */

	// OWL-S Process
	public String __OWLS_PROCESS = "http://www.daml.org/services/owl-s/1.1/Process.owl#process";

	public String __OWLS_ATOMIC_PROCESS = "http://www.daml.org/services/owl-s/1.1/Process.owl#AtomicProcess";

	public String __OWLS_COMPOSITE_PROCESS = "http://www.daml.org/services/owl-s/1.1/Process.owl#CompositeProcess";

	public String __OWLS_COMPOSED_OF = "http://www.daml.org/services/owl-s/1.1/Process.owl#composedOf";

	public String __OWLS_CONTROL_CONSTRUCT = "http://www.daml.org/services/owl-s/1.1/Process.owl#ControlConstruct";

	public String __OWLS_CONTROL_CONSTRUCT_LIST = "http://www.daml.org/services/owl-s/1.1/Process.owl#ControlConstructList";

	public String __OWLS_COMPONENTS = "http://www.daml.org/services/owl-s/1.1/Process.owl#components";

	public String __OWLS_HAS_DATA_FROM = "http://www.daml.org/services/owl-s/1.1/Process.owl#hasDataFrom";

	public String __OWLS_INPUT_BINDING = "http://www.daml.org/services/owl-s/1.1/Process.owl#InputBinding";

	public String __OWLS_OUTPUT_BINDING = "http://www.daml.org/services/owl-s/1.1/Process.owl#OutputBinding";

	public String __OWLS_VALUE_SOURCE = "http://www.daml.org/services/owl-s/1.1/Process.owl#valueSource";

	public String __OWLS_TO_PARAM = "http://www.daml.org/services/owl-s/1.1/Process.owl#toParam";

	public String __OWLS_THE_VAR = "http://www.daml.org/services/owl-s/1.1/Process.owl#theVar";

	public String __OWLS_FROM_PROCESS = "http://www.daml.org/services/owl-s/1.1/Process.owl#fromProcess";

	public String __HASOUTPUT_PROCESS_PROPERTY = "http://www.daml.org/services/owl-s/1.1/Process.owl#hasOutput";

	public String __HASINPUT_PROCESS_PROPERTY = "http://www.daml.org/services/owl-s/1.1/Process.owl#hasInput";

	public String __PARAMETERTYPE_PROPERTY = "http://www.daml.org/services/owl-s/1.1/Process.owl#parameterType";

	public String __PRESENTEDBY_PROPERTY = "http://www.daml.org/services/owl-s/1.1/Service.owl#presentedBy";

	public String __OWLS_PARENT_PROCESS = "http://www.daml.org/services/owl-s/1.1/Process.owl#TheParentPerform";

	// Specific capabilities
	public String __OWLS_CAPABILITY = "http://www.dyonisos.org/dyonisos.owl#Functionality";

	public String __OWLS_INPUT = "http://www.daml.org/services/owl-s/1.1/Process.owl#Input";

	public String __SERVICE_CAPABILITY = "http://www.dyonisos.org/functionalities.owl#ServiceFunctionality";

	// OWLS properties
	public String __HASOUTPUT_PROPERTY = "http://www.dyonisos.org/dyonisos.owl#hasOutput";

	public String __HASINPUT_PROPERTY = "http://www.dyonisos.org/dyonisos.owl#hasInput";

	public String __HASCAPABILITY_PROPERTY = "http://www.dyonisos.org/dyonisos.owl#hasFunctionality";

	public String __HASPROVIDEDCAPABILITY_PROPERTY = "http://www.dyonisos.org/dyonisos.owl#hasProvidedFunctionality";

	public String __HASREQUIREDCAPABILITY_PROPERTY = "http://www.dyonisos.org/dyonisos.owl#hasRequiredFunctionality";

	public String __HASCONVERSATION_PROPERTY = "http://www.dyonisos.org/dyonisos.owl#hasConversation";

	public String __SERVICE_TYPE = "http://amigo.gforge.inria.fr/owl/Amigo-S.owl#ServiceType";

	// OWL-S Service
	public String __OWLS_SUPPORTS = "http://www.daml.org/services/owl-s/1.1/Service.owl#supports";

	public String __OWLS_SERVICE = "http://www.daml.org/services/owl-s/1.1/Service.owl#Service";

	// public String __OWLS_SERVICEPROFILE =
	// "http://www.dyonisos.org/functionalities.owl#ServiceProfile";
	public String __OWLS_SERVICEPROFILE = "http://www.daml.org/services/owl-s/1.1/Service.owl#ServiceProfile";

	// OBJLIST
	public String __OBJLIST_FIRST = "http://www.daml.org/services/owl-s/1.1/generic/ObjectList.owl#first";

	public String __OBJLIST_REST = "http://www.daml.org/services/owl-s/1.1/generic/ObjectList.owl#rest";

	public String __OBJLIST_NIL = "http://www.daml.org/services/owl-s/1.1/generic/ObjectList.owl#nil";

	// RDF
	public String __RDF_TYPE = "http://www.w3.org/1999/02/22-rdf-syntax-ns#type";

	// OWL-S Grounding
	public String __OWLS_WSDL_ATOMIC_PROCESS_GROUNDING = "http://www.daml.org/services/owl-s/1.1/Grounding.owl#WsdlAtomicProcessGrounding";

	public String __OWLS_HAS_ATOMIC_PROCESS_GROUNDING = "http://www.daml.org/services/owl-s/1.1/Grounding.owl#hasAtomicProcessGrounding";

	public String __OWLS_GROUNDING_PROCESS = "http://www.daml.org/services/owl-s/1.1/Grounding.owl#owlsProcess";

	public String __OWLS_WSDL_DOCUMENT = "http://www.daml.org/services/owl-s/1.1/Grounding.owl#wsdlDocument";

	public String __OWLS_WSDL_OPERATION = "http://www.daml.org/services/owl-s/1.1/Grounding.owl#wsdlOperation";

	public String __OWLS_WSDL_OPERATION_REF = "http://www.daml.org/services/owl-s/1.1/Grounding.owl#WsdlOperationRef";

	public String __OWLS_GROUNDING_OPERATION = "http://www.daml.org/services/owl-s/1.1/Grounding.owl#operation";

	public String __OWLS_GROUNDING_PORTTYPE = "http://www.daml.org/services/owl-s/1.1/Grounding.owl#portType";

	public String __OWLS_WSDL_INPUTMESSAGE = "http://www.daml.org/services/owl-s/1.1/Grounding.owl#wsdlInputMessage";

	public String __OWLS_WSDL_OUTPUTMESSAGE = "http://www.daml.org/services/owl-s/1.1/Grounding.owl#wsdlOutputMessage";

	public String __OWLS_WSDL_INPUT = "http://www.daml.org/services/owl-s/1.1/Grounding.owl#wsdlInput";

	public String __OWLS_WSDL_OUTPUT = "http://www.daml.org/services/owl-s/1.1/Grounding.owl#wsdlOutput";

	public String __OWLS_WSDL_MESSAGE_PART = "http://www.daml.org/services/owl-s/1.1/Grounding.owl#wsdlMessagePart";

	public String __OWLS_GROUNDING_OWLS_PARAMETER = "http://www.daml.org/services/owl-s/1.1/Grounding.owl#owlsParameter";

	/* QoS */
	public String __OWLS_QOS_PROPERTY = "http://www.dyonisos.org/dyonisos.owl#QoSProperty";

	public String __OWLS_QOS_REQUIREMENT = "http://www.dyonisos.org/dyonisos.owl#QoSRequirement";

	public String __OWLS_QOS_HAS_FUNCTIONALITY_REQUIREMENT = "http://www.dyonisos.org/dyonisos.owl#hasFunctionalityLevelQoSRequirement";

	public String __OWLS_QOS_HAS_SUB_FUNCTIONALITY_PROPERTY = "http://www.dyonisos.org/dyonisos.owl#hasSubFunctionalityLevelQoSProperty";

	public String __OWLS_QOS_HAS_OPERATOR = "http://www.dyonisos.org/dyonisos.owl#hasOperator";

	public String __OWLS_QOS_HAS_QOS_PROPERTY = "http://www.dyonisos.org/dyonisos.owl#hasQoSProperty";

	public String __OWLS_QOS_HAS_VALUE = "http://www.dyonisos.org/dyonisos.owl#hasValue";

	public String __OWLS_QOS_HAS_TYPE = "http://www.dyonisos.org/dyonisos.owl#hasType";

	public String __HASQOSPROFILE_PROPERTY = "http://www.owl-ontologies.com/Amigo/Amigo.owl#hasQoSProfile";

	public String __QOSPROFILE = "http://www.owl-ontologies.com/Amigo/Amigo.owl#QoSProfile";

	/* main functionality ontology */
	public String __FUNCTIONALITY_NS = "http://www.dyonisos.org/functionalities.owl#";

	/* WSDL Parser */
	public final String WSDL_DEFINITIONS_TAG = "definitions";

	public final String WSDL_TARGETNAMESPACE_PROPERTY = "targetNamespace";

	public final String WSDL_SERVICE_TAG = "service";

	public final String WSDL_NAME_PROPERTY = "name";

	public final String WSDL_PORTTYPE_TAG = "portType";

	public final String WSDL_PORT_TAG = "port";

	public final String WSDL_MESSAGE_TAG = "message";

	public final String WSDL_SOAPADDRESS_TAG = "address";

	public final String WSDL_LOCATION_PROPERTY = "location";

	/* SAWSDL Parser */

	public final String SAWSDL_DESCRIPTION_TAG = "description";

	/* OWLS Parser */

	public final String OWLS_RDF_TAG = "RDF";

	public final String OWLS_BASE_PROPERTY = "xml:base";
	
	/**
	 * The separator used to separate the name of the field to its value in the
	 * file co ntaing the list of ontology used
	 */
	private static final String ONTOLOGY_SEPARATOR = " = ";

	/**
	 * The String that will be used to recognize comments in the file listing
	 * all ontology
	 */
	private static final String ONTOLOGY_COMMENTS = "#";

	
	/**
	 * The path of the file containing the set of concept that will be used for
	 * parsing
	 */
	private static final String ONTOLOGY_CONFIGURATIONFILE = ConfigManager.CONFIG_DIR
			+ ConfigManager.FILE_SEPARATOR + "ontology_for_parsing.config";

	

	// private static String prefix = JenaStarter.FILE_SEPARATOR+"ontologies";
	// private static String LOCALDIRECTORY = home + prefix;

	/*
	 * Correct a windows file URI into a correct URI
	 * 
	 * @param oldURI : the URI we need to correct (must start with "file:")
	 * @return the corrected URL
	 * 
	 * public String correctURI(String oldURI){ //we correct URI only if we are
	 * under windows if(FILE_SEPARATOR.equals("\\")){ String tmp =
	 * oldURI.substring(5); tmp = "file:/"+tmp.replace('\\','/').replace("
	 * ","%20"); //System.out.println("PropertyManager::correctURI(...)
	 * correctedURI = "+tmp); return tmp; } return oldURI; }
	 */

	/* **************************** */
	/* * Singloton Design Pattern * */
	/* **************************** */

	/** The global instance of the object */
	private static PropertyManager currentManager = null;

	/**
	 * Default Constructor of the Manager
	 * 
	 */
	private PropertyManager() {
		currentManager = this;

		// load the list of Ontology that will be used during the parsing from
		// the OntologyFile
		this.loadOntologyFile();

		// initialize other field thanks to the configuration file
		//this.initialize();

		// save the list of Ontology
		// this.saveOntologyInFile();
	}

	/**
	 * This method permit to get a reference of the singloton object containing
	 * all propertt needed for all process
	 * 
	 * @return the singloton object PropertyManager
	 */
	public static PropertyManager getInstance() {
		if (currentManager == null) {
			return new PropertyManager();
		} else {
			return currentManager;
		}
	}


	/**
	 * method to display all field of this class. The toString method cannot be
	 * overwrited because this create a invisible loop !
	 * 
	 * @return the description of the class
	 */
	public String myToString() {
		Class c = this.getClass();
		Field[] f = c.getFields();
		String result = "PropertyManager = " + f.length;
		for (int i = 0; i < f.length; i++) {
			try {
				result += "\n\t> " + f[i].getName() + "("
						+ Modifier.toString(f[i].getModifiers()) + ")" + " = "
						+ f[i].get(this);
			} catch (IllegalArgumentException e) {
				DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
			} catch (IllegalAccessException e) {
				DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
			}
		}
		return result;
	}

	/**
	 * This method save all field corresponding to an ontology concept (which
	 * name start with "__") in a file $ONTOLOGY_CONFIGURATIONFILE
	 * 
	 */
	@SuppressWarnings("unused")
	private void saveOntologyInFile() {
		Class c = this.getClass();
		Field[] f = c.getDeclaredFields();

		/* Generate the result string */
		String result = "#PropertyManager = " + f.length;
		for (int i = 0; i < f.length; i++) {
			if (f[i].getName().startsWith("__")) {
				try {
					result += "\n" + f[i].getName() + ONTOLOGY_SEPARATOR
							+ f[i].get(this);
				} catch (IllegalArgumentException e) {
					DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
				} catch (IllegalAccessException e) {
					DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
				}
			}
		}

		/* Store it in a file */
		BufferedWriter in = null;
		File file = null;
		try {
			file = new File(ONTOLOGY_CONFIGURATIONFILE);
			if (file.exists()) {
				file.delete();
			}
			file.createNewFile();
			in = new BufferedWriter(new FileWriter(file));
			in.write(result);

		} catch (SecurityException e) {
			DebugPrinter.print(file.getAbsolutePath() + " is not writable !", 2);
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
		} catch (IOException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
		} finally {
			if (in != null) {
				try {
					in.close();
				} catch (IOException e) {
					DebugPrinter.print(e.getStackTrace(), "(finally) "
							+ e.getMessage(), 2);
				}
			}
		}

	}

	/**
	 * This method save all field corresponding to an ontology concept (which
	 * name start with "__") from a file $ONTOLOGY_CONFIGURATIONFILE If the file
	 * doesn't exist, or is not readable, then the default value of field will
	 * be used.
	 * 
	 */
	private void loadOntologyFile() {

		File file = null;
		file = new File(ONTOLOGY_CONFIGURATIONFILE);
		if (file.exists()) {
			if (file.canRead()) {
				Class c = this.getClass();
				Field[] f = c.getDeclaredFields();

				/*
				 * prepare a treemap to find quickly the needed field (index on
				 * the name)
				 */
				TreeMap<String, Field> orderedArray = new TreeMap<String, Field>();
				for (int i = 0; i < f.length; i++) {
					orderedArray.put(f[i].getName(), f[i]);
				}

				/* Read the file, extract information and update the field */

				// creation of reader
				BufferedReader out = null;
				try {
					out = new BufferedReader(new FileReader(file));
					String currentLine = null;
					// read each line of the file
					while ((currentLine = out.readLine()) != null) {
						currentLine = currentLine.trim();
						// ignore empty line and comments, and line that didn't
						// contains the $ONTOLOGY_SEPARATOR
						if (!currentLine.startsWith(ONTOLOGY_COMMENTS)
								&& !(currentLine.length() == 0)
								&& currentLine.contains(ONTOLOGY_SEPARATOR)) {

							// extract informations
							String[] informations = currentLine.split(ONTOLOGY_SEPARATOR);
							try {
								String nameOfField = informations[0].trim();
								String valueOfField = informations[1].trim();

								// update the field
								Field toUpdate = orderedArray.get(nameOfField);
								if (toUpdate != null) {
									try {
										toUpdate.set(this, valueOfField);
									} catch (IllegalArgumentException e) {
										DebugPrinter.print(e.getStackTrace(),
											e.getMessage(), 2);
									} catch (IllegalAccessException e) {
										DebugPrinter.print(e.getStackTrace(),
											e.getMessage(), 2);
									}
								}

							} catch (IndexOutOfBoundsException e) {
								DebugPrinter.print(e.getStackTrace(),
									e.getMessage(), 2);
							}
						}
					}

				} catch (FileNotFoundException e) {
					DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
				} catch (IOException e) {
					DebugPrinter.print(e.getStackTrace(), e.getMessage(), 2);
				} finally {
					if (out != null) {
						try {
							out.close();
						} catch (IOException e) {
							DebugPrinter.print(e.getStackTrace(), "(finally) "
									+ e.getMessage(), 2);
						}
					}
				}

			} else {
				DebugPrinter.print(file.getAbsolutePath()
						+ " is not readable !", 2);
			}
		} else {
			DebugPrinter.print(file.getAbsolutePath() + " does not exist !", 2);
		}

		DebugPrinter.print("List of ontology loaded", 1);

	}

}
