package fr.inria.icocoa.servicemodel.parsers.jenaparser.tools;

import java.util.StringTokenizer;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.rdf.model.Statement;

import fr.inria.icocoa.exception.ParsingException;

/**
 * 
 * ExtendedTools.java<br>
 * This class contains some method usefull for the parser (that use Jena)
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Jul 16, 2007 4:24:16 PM
 */
public class ExtendedTools {

	/**
	 * This method convert a Statement into an Individual by extracting the
	 * Object argument of the Statement
	 * 
	 * @param model :
	 *            the ontology model from which the convertion is done
	 * @param state :
	 *            the statement we want to convert into an Individual
	 * @return the Individual mentioned into the Object field of the statement.
	 */
	public static Individual getIndividualFromStatement(OntModel model,
			Statement state) {
		return model.getIndividual(state.getObject().toString());
	}

	/**
	 * This method convert a Statement into a URI by extracting the Object
	 * Argument (Contain "ontology#Name Type"), then extract "ontology#Name"
	 * 
	 * @param state :
	 *            the state we want to obtain the URI
	 * @return the URI of the object contained into the statement
	 * @throws ParsingException
	 *             if an error occurs during the split process (see message
	 *             field)
	 */
	public static String getURIFromStatement(Statement state)
			throws ParsingException {

		try {
			String ontologyAndNameMerged = state.getObject().toString();

			StringTokenizer tok = new StringTokenizer(ontologyAndNameMerged);
			String ontologyAndName = tok.nextToken();

			// if you don't put a space after the declaration of the class of
			// Input/Output, a ^^nameOfCodage appear.
			// we need to remove it !
			if (ontologyAndName.contains("^^")) {
				StringTokenizer st = new StringTokenizer(ontologyAndName, "^^");
				ontologyAndName = st.nextToken();
			}
			return ontologyAndName;
		} catch (IndexOutOfBoundsException e) {
			throw new ParsingException("The statement " + state
					+ " doesn't contain an ontology. (More : " + e.getMessage()
					+ ")");
		}
	}

}
