/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel.parsers.jenaparser.tools;

import java.util.Map;

import fr.inria.icocoa.servicemodel.IO;

/**
 * functionalityMap.java
 * 
 * Mapping between a functionality ID and its semantic class<br>
 * functionalityMap = (String id,String type)
 * <p>
 * Example:
 * (http://www.dyonisos.org/MultimediaPlayer.owl#MediaRendererFunctionality,http://www.dyonisos.org/functionalities.owl#MediaRenderer)
 * 
 * @author <a href="mailto:Galip-Ferda.Tartanoglu@inria.fr">Ferda Tartanoglu</a> -
 *         <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 */
public class FunctionalityMap {

	/** The ID of the Functionality */
	private String id = "";

	/** The type of the Functionality */
	private String type = "";

	/** The matching table of IO */
	private Map<IO, IO> matchingTable = null;

	/**
	 * Constructor of the class
	 * 
	 * @param functionalityID :
	 *            The ID of the Functionality
	 * @param functionalityCLASS :
	 *            The type of the Functionality
	 */
	public FunctionalityMap(String functionalityID, String functionalityCLASS) {
		this.id = functionalityID;
		this.type = functionalityCLASS;
	}

	@Override
	public String toString() {
		if (matchingTable == null)
			return "(" + getId() + "," + getType() + ")";
		else
			return "(" + getId() + "," + getType() + "," + getMatchingTable()
					+ ")";
	}

	@Override
	public boolean equals(Object fm) {
		if (fm instanceof FunctionalityMap) {
			FunctionalityMap ffm = (FunctionalityMap) fm;
			return (this.getId().equals(ffm.getId()) && this.getType().equals(
				ffm.getType()));
		} else
			return false;
	}

	@Override
	public int hashCode() {
		return (this.getId().hashCode() + this.getType().hashCode())
				% Integer.MAX_VALUE;
	}

	/**
	 * @param id
	 *            the id to set
	 */
	public void setId(String id) {
		this.id = id;
	}

	/**
	 * @return the id
	 */
	public String getId() {
		return id;
	}

	/**
	 * @param type
	 *            the type to set
	 */
	public void setType(String type) {
		this.type = type;
	}

	/**
	 * @return the type
	 */
	public String getType() {
		return type;
	}

	/**
	 * @return the matchingTable
	 */
	public Map<IO, IO> getMatchingTable() {
		return matchingTable;
	}

	/**
	 * @param matchingTable
	 *            the matchingTable to set
	 */
	public void setMatchingTable(Map<IO, IO> matchingTable) {
		this.matchingTable = matchingTable;
	}

}
