/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.junittest.servicemodel.matching;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import fr.inria.icocoa.exception.SemanticTypeNotKnowException;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.Input;
import fr.inria.icocoa.servicemodel.Output;
import fr.inria.icocoa.servicemodel.matching.CapabilityMatch;
import fr.inria.icocoa.servicemodel.matching.Matcher;
import fr.inria.icocoa.servicemodel.matching.SyntacticMatcher;
import fr.inria.icocoa.servicemodel.matching.WeakSemanticMatcher;

/**
 * CapabilityMatchTest.java
 *
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Aug 29, 2008 11:24:31 AM
 */
public class CapabilityMatchTest {

	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {
	}

	/**
	 * Test method for {@link fr.inria.icocoa.servicemodel.matching.CapabilityMatch#isExact()}.
	 * @throws SemanticTypeNotKnowException 
	 */
	@Test
	public final void testIsExact() throws SemanticTypeNotKnowException {
//		 Reference Capa
		Input in1 = new Input("in1",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in2 = new Input("in2",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in3 = new Input("in3",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out1 = new Output("out1",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out2 = new Output("out2",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out3 = new Output("out3",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa1 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa1");

		capa1.addInput(in1);
		capa1.addInput(in2);
		capa1.addInput(in3);

		capa1.addOutput(out1);
		capa1.addOutput(out2);
		capa1.addOutput(out3);

		// Other Capa
		Input in11 = new Input("in11",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in21 = new Input("in21",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in31 = new Input("in31",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out11 = new Output("out11",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out21 = new Output("out21",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out31 = new Output("out31",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa11 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa11");

		capa11.addInput(in11);
		capa11.addInput(in21);
		capa11.addInput(in31);

		capa11.addOutput(out11);
		capa11.addOutput(out21);
		capa11.addOutput(out31);
		
		
		//Other Capa
		Input in12 = new Input("in12",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in22 = new Input("in22",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in32 = new Input("in32",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out12 = new Output("out12",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out22 = new Output("out22",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");

		Capability capa12 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa12");

		capa12.addInput(in12);
		capa12.addInput(in22);
		capa12.addInput(in32);

		capa12.addOutput(out12);
		capa12.addOutput(out22);
		
		//Other Capa
		Input in13 = new Input("in13",
				"http://www.inria.fr/myPiOntology.owl#SemInChild1");
		Input in23 = new Input("in23",
				"http://www.inria.fr/myPiOntology.owl#SemIn21");
		Input in33 = new Input("in33",
				"http://www.inria.fr/myPiOntology.owl#SemInChildChild3");

		Output out13 = new Output("out13",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out23 = new Output("out23",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out33 = new Output("out33",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa13 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa13");

		capa13.addInput(in13);
		capa13.addInput(in23);
		capa13.addInput(in33);

		capa13.addOutput(out13);
		capa13.addOutput(out23);
		capa13.addOutput(out33);
		
//		 other Capa
		Input in14 = new Input("in14",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in24 = new Input("in24",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in34 = new Input("in34",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out14 = new Output("out14",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out24 = new Output("out24",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out34 = new Output("out34",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa4 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapaChild1", null, "capa4");

		capa4.addInput(in14);
		capa4.addInput(in24);
		capa4.addInput(in34);

		capa4.addOutput(out14);
		capa4.addOutput(out24);
		capa4.addOutput(out34);
		

		

		CapabilityMatch matching = new SyntacticMatcher().capabilityMatch(capa1,capa11); 
		assertTrue("exact match ok",matching.isExact());
		
		Matcher matcher = new WeakSemanticMatcher();
		
		CapabilityMatch matching2 = matcher.capabilityMatch(capa1,capa11); 		
		assertTrue("exact match ok 2 ",matching2.isExact());
		
		CapabilityMatch matching3 = matcher.capabilityMatch(capa1,capa12); 		
		assertNull("no match",matching3);
		
		CapabilityMatch matching4 = matcher.capabilityMatch(capa1,capa13);
		assertFalse("not exact match",matching4.isExact());
		
		CapabilityMatch matching5 = matcher.capabilityMatch(capa1,capa4);
		assertFalse("not exact match (inclusive)",matching5.isExact());
		
		CapabilityMatch matching6 = matcher.capabilityMatch(capa4,capa1);
		assertFalse("not exact match (not inclusive)",matching6.isExact());
		
		
	}

	/**
	 * Test method for {@link fr.inria.icocoa.servicemodel.matching.CapabilityMatch#isInclusive()}.
	 * @throws SemanticTypeNotKnowException 
	 */
	@Test
	public final void testIsInclusive() throws SemanticTypeNotKnowException {
//		 Reference Capa
		Input in1 = new Input("in1",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in2 = new Input("in2",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in3 = new Input("in3",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out1 = new Output("out1",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out2 = new Output("out2",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out3 = new Output("out3",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa1 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa1");

		capa1.addInput(in1);
		capa1.addInput(in2);
		capa1.addInput(in3);

		capa1.addOutput(out1);
		capa1.addOutput(out2);
		capa1.addOutput(out3);

		// Other Capa
		Input in11 = new Input("in11",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in21 = new Input("in21",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in31 = new Input("in31",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out11 = new Output("out11",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out21 = new Output("out21",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out31 = new Output("out31",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa11 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa11");

		capa11.addInput(in11);
		capa11.addInput(in21);
		capa11.addInput(in31);

		capa11.addOutput(out11);
		capa11.addOutput(out21);
		capa11.addOutput(out31);
		
		
		//Other Capa
		Input in12 = new Input("in12",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in22 = new Input("in22",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in32 = new Input("in32",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out12 = new Output("out12",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out22 = new Output("out22",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");

		Capability capa12 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa12");

		capa12.addInput(in12);
		capa12.addInput(in22);
		capa12.addInput(in32);

		capa12.addOutput(out12);
		capa12.addOutput(out22);
		
		//Other Capa
		Input in13 = new Input("in13",
				"http://www.inria.fr/myPiOntology.owl#SemInChild1");
		Input in23 = new Input("in23",
				"http://www.inria.fr/myPiOntology.owl#SemIn21");
		Input in33 = new Input("in33",
				"http://www.inria.fr/myPiOntology.owl#SemInChildChild3");

		Output out13 = new Output("out13",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out23 = new Output("out23",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out33 = new Output("out33",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa13 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa13");

		capa13.addInput(in13);
		capa13.addInput(in23);
		capa13.addInput(in33);

		capa13.addOutput(out13);
		capa13.addOutput(out23);
		capa13.addOutput(out33);
		
//		 other Capa
		Input in14 = new Input("in14",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in24 = new Input("in24",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in34 = new Input("in34",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out14 = new Output("out14",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out24 = new Output("out24",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out34 = new Output("out34",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa4 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapaChild1", null, "capa4");

		capa4.addInput(in14);
		capa4.addInput(in24);
		capa4.addInput(in34);

		capa4.addOutput(out14);
		capa4.addOutput(out24);
		capa4.addOutput(out34);
		

		

		CapabilityMatch matching = new SyntacticMatcher().capabilityMatch(capa1,capa11); 
		assertTrue("exact match ok",matching.isInclusive());
		
		Matcher matcher = new WeakSemanticMatcher();
		
		CapabilityMatch matching2 = matcher.capabilityMatch(capa1,capa11); 		
		assertTrue("exact match ok 2 ",matching2.isInclusive());
		
		CapabilityMatch matching3 = matcher.capabilityMatch(capa1,capa12); 		
		assertNull("no match",matching3);
		
		CapabilityMatch matching4 = matcher.capabilityMatch(capa1,capa13);
		assertTrue("exact match",matching4.isInclusive());
		
		CapabilityMatch matching5 = matcher.capabilityMatch(capa1,capa4);
		assertTrue("exact match (inclusive)",matching5.isInclusive());
		
		CapabilityMatch matching6 = matcher.capabilityMatch(capa4,capa1);
		assertFalse("not exact match (not inclusive)",matching6.isInclusive());
	}

}
