/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.junittest.servicemodel.matching;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import fr.inria.icocoa.exception.SemanticTypeNotKnowException;
import fr.inria.icocoa.servicemodel.Input;
import fr.inria.icocoa.servicemodel.Output;
import fr.inria.icocoa.servicemodel.matching.Matcher;
import fr.inria.icocoa.servicemodel.matching.WeakSemanticMatcher;

/**
 * IOMatchTest.java
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Sep 2, 2008 2:28:17 PM
 */
public class IOMatchTest {

	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.matching.IOMatch#isExact()}.
	 * 
	 * @throws SemanticTypeNotKnowException
	 */
	@Test
	public final void testIsExact() throws SemanticTypeNotKnowException {
		Input in1 = new Input("in1",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in2 = new Input("in2",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in3 = new Input("in3",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");
		Input in4 = new Input("in4",
				"http://www.inria.fr/myPiOntology.owl#SemInChild1");

		Output out1 = new Output("out1",
				"http://www.inria.fr/myPiOntology.owl#SemOutChild1");
		Output out2 = new Output("out2",
				"http://www.inria.fr/myPiOntology.owl#SemOut21");
		Output out3 = new Output("out3",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");
		Output out4 = new Output("out4",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		// Other Capa
		Input in11 = new Input("in11",
				"http://www.inria.fr/myPiOntology.owl#SemInChild1");
		Input in21 = new Input("in21",
				"http://www.inria.fr/myPiOntology.owl#SemIn21");
		Input in31 = new Input("in31",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");
		Input in41 = new Input("in41",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");

		Output out11 = new Output("out11",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out21 = new Output("out21",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out31 = new Output("out31",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");
		Output out41 = new Output("out41",
				"http://www.inria.fr/myPiOntology.owl#SemOutChild3");

		Matcher matcher = new WeakSemanticMatcher();

		assertFalse("in1<->in11", matcher.ioMatch(in1, in11).isExact());
		assertTrue("in2<->in21", matcher.ioMatch(in2, in21).isExact());
		assertTrue("in3<->in31", matcher.ioMatch(in3, in31).isExact());
		assertFalse("in4<->in41", matcher.ioMatch(in4, in41).isExact());

		assertFalse("out1<->out11", matcher.ioMatch(out1, out11).isExact());
		assertTrue("out2<->out21", matcher.ioMatch(out2, out21).isExact());
		assertTrue("out3<->out31", matcher.ioMatch(out3, out31).isExact());
		assertFalse("out4<->out41", matcher.ioMatch(out4, out41).isExact());
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.matching.IOMatch#isInclusive()}.
	 * 
	 * @throws SemanticTypeNotKnowException
	 */
	@Test
	public final void testIsInclusive() throws SemanticTypeNotKnowException {
		Input in1 = new Input("in1",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in2 = new Input("in2",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in3 = new Input("in3",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");
		Input in4 = new Input("in4",
				"http://www.inria.fr/myPiOntology.owl#SemInChild1");

		Output out1 = new Output("out1",
				"http://www.inria.fr/myPiOntology.owl#SemOutChild1");
		Output out2 = new Output("out2",
				"http://www.inria.fr/myPiOntology.owl#SemOut21");
		Output out3 = new Output("out3",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");
		Output out4 = new Output("out4",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		// Other Capa
		Input in11 = new Input("in11",
				"http://www.inria.fr/myPiOntology.owl#SemInChild1");
		Input in21 = new Input("in21",
				"http://www.inria.fr/myPiOntology.owl#SemIn21");
		Input in31 = new Input("in31",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");
		Input in41 = new Input("in41",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");

		Output out11 = new Output("out11",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out21 = new Output("out21",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out31 = new Output("out31",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");
		Output out41 = new Output("out41",
				"http://www.inria.fr/myPiOntology.owl#SemOutChild3");

		Matcher matcher = new WeakSemanticMatcher();

		assertTrue("in1<->in11", matcher.ioMatch(in1, in11).isInclusive());
		assertTrue("in2<->in21", matcher.ioMatch(in2, in21).isInclusive());
		assertTrue("in3<->in31", matcher.ioMatch(in3, in31).isInclusive());
		assertFalse("in4<->in41", matcher.ioMatch(in4, in41).isInclusive());

		assertTrue("out1<->out11", matcher.ioMatch(out1, out11).isInclusive());
		assertTrue("out2<->out21", matcher.ioMatch(out2, out21).isInclusive());
		assertTrue("out3<->out31", matcher.ioMatch(out3, out31).isInclusive());
		assertFalse("out4<->out41", matcher.ioMatch(out4, out41).isInclusive());
	}

}
