/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.junittest.servicemodel.parsers.tools;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

import fr.inria.icocoa.servicemodel.toolsobjects.CapabilityMap;

/**
 * CapabilityMapTest.java
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : May 14, 2007 2:17:33 PM
 */
public class CapabilityMapTest {

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.toolsobjects.CapabilityMap#hashCode()}.
	 */
	@Test
	public final void testHashCode() {
		CapabilityMap cm1 = new CapabilityMap("toto", "titi", "tata");
		CapabilityMap cm2 = new CapabilityMap("toto", "titi", "tata");
		CapabilityMap cm3 = new CapabilityMap("toto2", "titi", "tata");
		CapabilityMap cm4 = new CapabilityMap("toto", "titi2", "tata");
		CapabilityMap cm5 = new CapabilityMap("toto", "titi", "tata2");

		// equals
		assertTrue("testEqualsObject : same object", cm1.hashCode() == cm1
				.hashCode());
		assertTrue("testEqualsObject : same semantic", cm1.hashCode() == cm2
				.hashCode());

		// not equals
		assertFalse("testEqualsObject : not same semantic 1",
				cm1.hashCode() == cm3.hashCode());
		assertFalse("testEqualsObject : not same semantic 2",
				cm1.hashCode() == cm4.hashCode());
		assertFalse("testEqualsObject : not same semantic 3",
				cm1.hashCode() == cm5.hashCode());
		assertFalse("testEqualsObject : not same semantic 4",
				cm3.hashCode() == cm1.hashCode());
		assertFalse("testEqualsObject : not same semantic 5",
				cm3.hashCode() == cm4.hashCode());
		assertFalse("testEqualsObject : not same semantic 6",
				cm3.hashCode() == cm5.hashCode());
		assertFalse("testEqualsObject : not same semantic 7",
				cm4.hashCode() == cm1.hashCode());
		assertFalse("testEqualsObject : not same semantic 8",
				cm4.hashCode() == cm3.hashCode());
		assertFalse("testEqualsObject : not same semantic 9",
				cm4.hashCode() == cm5.hashCode());
		assertFalse("testEqualsObject : not same semantic 10",
				cm5.hashCode() == cm1.hashCode());
		assertFalse("testEqualsObject : not same semantic 11",
				cm5.hashCode() == cm3.hashCode());
		assertFalse("testEqualsObject : not same semantic 12",
				cm5.hashCode() == cm4.hashCode());
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.toolsobjects.CapabilityMap#equals(java.lang.Object)}.
	 */
	@Test
	public final void testEqualsObject() {
		CapabilityMap cm1 = new CapabilityMap("toto", "titi", "tata");
		CapabilityMap cm2 = new CapabilityMap("toto", "titi", "tata");
		CapabilityMap cm3 = new CapabilityMap("toto2", "titi", "tata");
		CapabilityMap cm4 = new CapabilityMap("toto", "titi2", "tata");
		CapabilityMap cm5 = new CapabilityMap("toto", "titi", "tata2");

		// equals
		assertTrue("testEqualsObject : same object", cm1.equals(cm1));
		assertTrue("testEqualsObject : same semantic", cm1.equals(cm2));
		assertTrue("testEqualsObject : same reference", cm1 == cm1);

		// not equals
		assertTrue("testEqualsObject : not same reference", cm1 != cm2);
		assertFalse("testEqualsObject : not same semantic 1", cm1.equals(cm3));
		assertFalse("testEqualsObject : not same semantic 2", cm1.equals(cm4));
		assertFalse("testEqualsObject : not same semantic 3", cm1.equals(cm5));
		assertFalse("testEqualsObject : not same semantic 4", cm3.equals(cm1));
		assertFalse("testEqualsObject : not same semantic 5", cm3.equals(cm4));
		assertFalse("testEqualsObject : not same semantic 6", cm3.equals(cm5));
		assertFalse("testEqualsObject : not same semantic 7", cm4.equals(cm1));
		assertFalse("testEqualsObject : not same semantic 8", cm4.equals(cm3));
		assertFalse("testEqualsObject : not same semantic 9", cm4.equals(cm5));
		assertFalse("testEqualsObject : not same semantic 10", cm5.equals(cm1));
		assertFalse("testEqualsObject : not same semantic 11", cm5.equals(cm3));
		assertFalse("testEqualsObject : not same semantic 12", cm5.equals(cm4));

	}

}
