/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.junittest.servicemodel;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import junit.framework.JUnit4TestAdapter;

import org.junit.Test;

import fr.inria.icocoa.servicemodel.IO;
import fr.inria.icocoa.servicemodel.Output;

/**
 * OutputTest.java<br>
 * 
 * This class contains tests for the class
 * {@link fr.inria.icocoa.servicemodel.Output}
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Apr 17, 2007
 */
public class OutputTest {

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Input#equals(java.lang.Object)}.
	 */
	@Test
	public final void testEqualsObject() {
		Output in = new Output("toto", "titi", "tata");
		Output in2 = new Output("toto", "titi", "tata");
		Output in3 = new Output("toto2", "titi", "tata");
		Output in4 = new Output("toto", "titi2", "tata");
		Output in5 = new Output("toto", "titi", "tata2");

		// equals
		assertEquals("testEqualsObject : same object", true, in.equals(in));
		assertEquals("testEqualsObject : same semantic", true, in.equals(in2));
		assertEquals("testEqualsObject : same reference", true, in == in);

		// not equals
		assertEquals("testEqualsObject : not same reference", true, in != in2);
		assertEquals("testEqualsObject : not same semantic 1", false, in
				.equals(in3));
		assertEquals("testEqualsObject : not same semantic 2", false, in
				.equals(in4));
		assertEquals("testEqualsObject : not same semantic 3", false, in
				.equals(in5));
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Output#Output(java.lang.String)}.
	 */
	@Test
	public final void testOutputString() {
		Output in = new Output("toto");
		Output in2 = new Output("toto", IO.UNKNOWN, IO.UNKNOWN);
		assertEquals("testOutputString : same semantic", true, in.equals(in2));
	}

	/**
	 * Test method for {@link fr.inria.icocoa.servicemodel.IO#hashCode()}.
	 */
	@Test
	public final void testHashCode() {
		Output in = new Output("toto", "titi", "tata");
		Output in2 = new Output("toto", "titi", "tata");
		assertEquals("testHashCode : same hashCode", true, in.hashCode() == in2
				.hashCode());

		Output in3 = new Output("toto", "tata", "titi");
		Output in4 = new Output("toto", "titi", "tata");
		assertEquals("testHashCode : different hashCode", false,
				in3.hashCode() == in4.hashCode());

		Output in5 = new Output("toto", "titi", null);
		Output in6 = new Output("toto", "titi", null);
		assertEquals("testHashCode : same hashCode 2", true,
				in5.hashCode() == in6.hashCode());
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Output#transfertInformation(fr.inria.icocoa.servicemodel.Output)}.
	 */
	@Test
	public final void testTransfertInformation() {

		Output a = new Output("a", "a#a");
		Output b = new Output("mylocalName", "myOntoilogy#myConceptName");
		b.setProvided(true);
		b.setGroundingValue("titi");

		assertFalse("Output not equals", a.equals(b));

		a.transfertInformation(b);
		assertTrue("Output equals", a.equals(b));
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.IO#countNullOrUnknowField()}.
	 */
	@Test
	public final void testCountNullOrUnknowField() {
		assertEquals("test Output 1", 1, new Output("a", "a#a")
				.countNullOrUnknowField());

		Output a = new Output("a", "a#a");
		a.setLocalName(IO.UNKNOWN);
		a.setType(IO.UNKNOWN + "#" + IO.UNKNOWN);
		assertEquals("test Output 4", 4, a.countNullOrUnknowField());

		Output b = new Output("a", "a#a");
		b.setGroundingValue("toto");
		assertEquals("test Output 0", 0, b.countNullOrUnknowField());

		Output c = new Output("a", "a#a");
		c.setLocalName(IO.UNKNOWN);
		assertEquals("test Output 2", 2, c.countNullOrUnknowField());

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Output#containsMoreInformation(fr.inria.icocoa.servicemodel.Output)}.
	 */
	@Test
	public final void testContainsMoreInformation() {
		assertEquals("test Output 0", 0, new Output("a", "a#a")
				.containsMoreInformation(new Output("a", "a#a")));

		Output a = new Output("a", "a#a");
		a.setLocalName(IO.UNKNOWN);
		a.setType(IO.UNKNOWN + "#" + IO.UNKNOWN);
		assertTrue("test Output <0", a.containsMoreInformation(new Output("a",
				"a#a")) < 0);
		assertTrue("test Output >0", new Output("a", "a#a")
				.containsMoreInformation(a) > 0);
	}

	/**
	 * Generate the suite of Test of Output class
	 * 
	 * @return a Test object
	 */
	public static junit.framework.Test suite() {
		return new JUnit4TestAdapter(OutputTest.class);
	}

}
