/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.junittest.servicemodel;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

import fr.inria.icocoa.servicemodel.ParameterBinding;

/**
 * ParameterBindingTest.java<br>
 * 
 * This class contains tests for the class
 * {@link fr.inria.icocoa.servicemodel.ParameterBinding}
 * 
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Apr 19, 2007 1:38:31 PM
 */
public class ParameterBindingTest {

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.ParameterBinding#hashCode()}.
	 */
	@Test
	public final void testHashCode() {
		ParameterBinding pb1 = new ParameterBinding("toto", "titi", "tutu",
				true);
		ParameterBinding pb2 = new ParameterBinding("toto", "titi", "tutu",
				true);
		ParameterBinding pb3 = new ParameterBinding("toto2", "titi", "tutu",
				true);
		ParameterBinding pb4 = new ParameterBinding("toto", "titi2", "tutu",
				true);
		ParameterBinding pb5 = new ParameterBinding("toto", "titi", "tutu3",
				true);
		ParameterBinding pb6 = new ParameterBinding("toto", "titi", "tutu",
				false);

		// same
		assertTrue("same obj", pb1.hashCode() == pb1.hashCode());
		assertTrue("same sem", pb1.hashCode() == pb2.hashCode());

		// not same
		assertFalse("pb1 vs pb3", pb1.hashCode() == pb3.hashCode());
		assertFalse("pb1 vs pb4", pb1.hashCode() == pb4.hashCode());
		assertFalse("pb1 vs pb5", pb1.hashCode() == pb5.hashCode());
		assertFalse("pb1 vs pb6", pb1.hashCode() == pb6.hashCode());
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.ParameterBinding#equals(java.lang.Object)}.
	 */
	@Test
	public final void testEqualsObject() {
		ParameterBinding pb1 = new ParameterBinding("toto", "titi", "tutu",
				true);
		ParameterBinding pb2 = new ParameterBinding("toto", "titi", "tutu",
				true);
		ParameterBinding pb3 = new ParameterBinding("toto2", "titi", "tutu",
				true);
		ParameterBinding pb4 = new ParameterBinding("toto", "titi2", "tutu",
				true);
		ParameterBinding pb5 = new ParameterBinding("toto", "titi", "tutu3",
				true);
		ParameterBinding pb6 = new ParameterBinding("toto", "titi", "tutu",
				false);

		// same
		assertTrue("same obj", pb1.equals(pb1));
		assertTrue("same ref", pb1 == pb1);
		assertTrue("same sem", pb1.equals(pb2));
		assertTrue("same sem 2", pb2.equals(pb1));

		// not same
		assertFalse("pb1 vs pb3", pb1.equals(pb3));
		assertFalse("pb3 vs pb1", pb3.equals(pb1));
		assertFalse("pb1 vs pb4", pb1.equals(pb4));
		assertFalse("pb4 vs pb1", pb4.equals(pb1));
		assertFalse("pb1 vs pb5", pb1.equals(pb5));
		assertFalse("pb5 vs pb1", pb5.equals(pb1));
		assertFalse("pb1 vs pb6", pb1.equals(pb6));
		assertFalse("pb6 vs pb1", pb6.equals(pb1));
	}

}
