/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.junittest.servicemodel;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;

import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.exception.CapabilityAlreadyRegisteredException;
import fr.inria.icocoa.exception.SameURIException;
import fr.inria.icocoa.exception.SemanticTypeNotKnowException;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.Input;
import fr.inria.icocoa.servicemodel.Output;
import fr.inria.icocoa.servicemodel.ParameterBinding;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.UserTask;
import fr.inria.icocoa.servicemodel.matching.CapabilityMatch;
import fr.inria.icocoa.servicemodel.matching.IOMatch;
import fr.inria.icocoa.servicemodel.matching.Matcher;
import fr.inria.icocoa.servicemodel.matching.MatchingType;
import fr.inria.icocoa.servicemodel.matching.ServiceMatch;
import fr.inria.icocoa.servicemodel.matching.SyntacticMatcher;

/**
 * ServiceTest.java<br>
 * 
 * This class contains tests for the class
 * {@link fr.inria.icocoa.servicemodel.Service} (no Capability)
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Apr 19, 2007 10:39:41 AM
 */
public class ServiceTest {

	/**
	 * Test method for {@link fr.inria.icocoa.servicemodel.Service#hashCode()}.
	 */
	@Test
	public final void testHashCode() {
		Service s1 = new Service();
		Service s2 = new Service();
		Service s3 = new Service();
		Service s4 = new Service();
		Service s5 = new Service();
		UserTask u5 = new UserTask(s5);
		Service s6 = new Service();
		Service s7 = new Service();
		Service s8 = new Service();

		s3.setBase("toto");
		s4.setLocalName("toto");
		s6.setWsdlDescriptionFile("toto");
		s7.setOwlDescriptionFile("toto");
		s8.setType("toto");

		// test equals
		assertTrue("same object", s1.hashCode() == s1.hashCode());
		assertTrue("same sem", s1.hashCode() == s2.hashCode());

		// test not equals
		assertFalse("s1 vs s3", s1.hashCode() == s3.hashCode());
		assertFalse("s1 vs s4", s1.hashCode() == s4.hashCode());
		assertFalse("s1 vs s6", s1.hashCode() == s6.hashCode());
		assertFalse("s1 vs s7", s1.hashCode() == s7.hashCode());
		assertFalse("s1 vs s8", s1.hashCode() == s8.hashCode());
		assertFalse("u5 vs s5", u5.hashCode() == s5.hashCode());

		// not same type
		String toto = "toto";
		assertFalse("Not same type", s1.equalsWithoutCapabilities(toto));

		/*
		 * Test ParameterBinding
		 */
		ParameterBinding b1 = new ParameterBinding("tata", "titi", "toto", true);
		ParameterBinding b2 = new ParameterBinding("tata", "titi", "toto", true);
		ParameterBinding b3 = new ParameterBinding("tata2", "titi", "toto",
				true);

		Service s9 = new Service("toto");
		Service s10 = new Service("toto");
		Service s11 = new Service("toto");
		Service s12 = new Service("toto");
		Service s13 = new Service("toto");

		s9.addParameterBinding(b1);
		s10.addParameterBinding(b1);
		s11.addParameterBinding(b2);
		s12.addParameterBinding(b3);
		s13.addParameterBinding(b1);
		s13.addParameterBinding(b3);

		// test equals
		assertTrue("same object db", s9.hashCode() == s10.hashCode());
		assertTrue("same sem db", s9.hashCode() == s11.hashCode());

		// test not equals
		assertFalse("not same obj db", s9.hashCode() == s12.hashCode());
		assertFalse("not same nb of db", s9.hashCode() == s13.hashCode());
	}

	/**
	 * Test method for {@link fr.inria.icocoa.servicemodel.Service#hashCode()}.
	 * (with Capability)
	 */
	@Test
	public final void testHashCodeWithCapa() {
		Service s1 = new Service("toot");
		Service s2 = new Service("toot");
		Service s3 = new Service("toot");
		Service s4 = new Service("toot");
		Service s5 = new Service("toot");

		Capability c1 = new Capability("toto#toto");
		Capability c2 = new Capability("toto#toto");
		Capability c3 = new Capability("titi#titi");

		c1.setCapabilityURI("blabla");
		c2.setCapabilityURI("blabla");
		c3.setCapabilityURI("blibli");

		s1.addCapability(c1);
		s2.addCapability(c1);
		s3.addCapability(c2);
		s4.addCapability(c3);
		s5.addCapability(c1);
		s5.addCapability(c3);

		// true
		assertTrue("same capa", s1.hashCode() == s2.hashCode());
		assertTrue("capa same semantic", s1.hashCode() == s3.hashCode());

		// false
		assertFalse("not same Capa", s1.hashCode() == s4.hashCode());
		assertFalse("not same Capa number", s1.hashCode() == s5.hashCode());
	}

	/**
	 * Test method for {@link fr.inria.icocoa.servicemodel.Service#hashCode()}.
	 * (with Required Capability)
	 */
	@Test
	public final void testHashCodeWithRequiredCapa() {
		Service s1 = new Service("toot");
		Service s2 = new Service("toot");
		Service s3 = new Service("toot");
		Service s4 = new Service("toot");
		Service s5 = new Service("toot");

		Capability c1 = new Capability("toto#toto");
		Capability c2 = new Capability("toto#toto");
		Capability c3 = new Capability("titi#titi");

		c1.setCapabilityURI("blabla");
		c2.setCapabilityURI("blabla");
		c3.setCapabilityURI("blibli");

		s1.addRequiredCapability(c1);
		s2.addRequiredCapability(c1);
		s3.addRequiredCapability(c2);
		s4.addRequiredCapability(c3);
		s5.addRequiredCapability(c1);
		s5.addRequiredCapability(c3);

		// true
		assertTrue("same capa", s1.hashCode() == s2.hashCode());
		assertTrue("capa same semantic", s1.hashCode() == s3.hashCode());

		// false
		assertFalse("not same Capa", s1.hashCode() == s4.hashCode());
		assertFalse("not same Capa number", s1.hashCode() == s5.hashCode());
	}

	/**
	 * Test method for {@link fr.inria.icocoa.servicemodel.Service#hashCode()}.
	 * (with Provided Capability)
	 */
	@Test
	public final void testHashCodeWithProvidedCapa() {
		Service s1 = new Service("toot");
		Service s2 = new Service("toot");
		Service s3 = new Service("toot");
		Service s4 = new Service("toot");
		Service s5 = new Service("toot");

		Capability c1 = new Capability("toto#toto");
		Capability c2 = new Capability("toto#toto");
		Capability c3 = new Capability("titi#titi");

		c1.setCapabilityURI("blabla");
		c2.setCapabilityURI("blabla");
		c3.setCapabilityURI("blibli");

		s1.addProvidedCapability(c1);
		s2.addProvidedCapability(c1);
		s3.addProvidedCapability(c2);
		s4.addProvidedCapability(c3);
		s5.addProvidedCapability(c1);
		s5.addProvidedCapability(c3);

		// true
		assertTrue("same capa", s1.hashCode() == s2.hashCode());
		assertTrue("capa same semantic", s1.hashCode() == s3.hashCode());

		// false
		assertFalse("not same Capa", s1.hashCode() == s4.hashCode());
		assertFalse("not same Capa number", s1.hashCode() == s5.hashCode());
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#equalsWithoutCapabilities(java.lang.Object)}.
	 */
	@Test
	public final void testEqualsWithoutCapabilitiesObject() {

		Service s1 = new Service();
		Service s2 = new Service();
		Service s3 = new Service();
		Service s4 = new Service();
		Service s5 = new Service();
		UserTask u5 = new UserTask(s5);
		Service s6 = new Service();
		Service s7 = new Service();
		Service s8 = new Service();

		s3.setBase("toto");
		s4.setLocalName("toto");
		s6.setWsdlDescriptionFile("toto");
		s7.setOwlDescriptionFile("toto");
		s8.setType("toto");

		// test equals
		assertTrue("same object", s1.equals(s1));
		assertTrue("same ref", s1 == s1);
		assertTrue("same sem", s1.equals(s2));
		assertTrue("same sem 2", s2.equals(s1));

		// test not equals
		assertFalse("s1 vs s3", s1.equals(s3));
		assertFalse("s3 vs s1", s3.equals(s1));
		assertFalse("s1 vs s4", s1.equals(s4));
		assertFalse("s4 vs s1", s4.equals(s1));
		assertFalse("s1 vs s6", s1.equals(s6));
		assertFalse("s6 vs s1", s6.equals(s1));
		assertFalse("s1 vs s7", s1.equals(s7));
		assertFalse("s7 vs s1", s7.equals(s1));
		assertFalse("s1 vs s8", s1.equals(s8));
		assertFalse("s8 vs s1", s8.equals(s1));
		assertFalse("s5 vs u5", s5.equals(u5));
		assertFalse("u5 vs s5", u5.equals(s5));

		// not same type
		String toto = "toto";
		assertFalse("Not same type", s1.equalsWithoutCapabilities(toto));

		/*
		 * Test ParameterBinding
		 */
		ParameterBinding b1 = new ParameterBinding("tata", "titi", "toto", true);
		ParameterBinding b2 = new ParameterBinding("tata", "titi", "toto", true);
		ParameterBinding b3 = new ParameterBinding("tata2", "titi", "toto",
				true);

		Service s9 = new Service("toto");
		Service s10 = new Service("toto");
		Service s11 = new Service("toto");
		Service s12 = new Service("toto");
		Service s13 = new Service("toto");

		s9.addParameterBinding(b1);
		s10.addParameterBinding(b1);
		s11.addParameterBinding(b2);
		s12.addParameterBinding(b3);
		s13.addParameterBinding(b1);
		s13.addParameterBinding(b3);

		// test equals
		assertTrue("same object db", s9.equals(s10));
		assertTrue("same object db 2", s10.equals(s9));
		assertTrue("same sem db", s9.equals(s11));
		assertTrue("same sem db 2", s11.equals(s9));

		// test not equals
		assertFalse("not same obj db", s9.equals(s12));
		assertFalse("not same obj db 2", s12.equals(s9));
		assertFalse("not same nb of db", s9.equals(s13));
		assertFalse("not same nb of db 2", s13.equals(s9));

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#equals(java.lang.Object)}.
	 */
	@Test
	public final void testEqualsObjectWithCapability() {

		/*
		 * Test Capability
		 */
		Service s1 = new Service("toot");
		Service s2 = new Service("toot");
		Service s3 = new Service("toot");
		Service s4 = new Service("toot");
		Service s5 = new Service("toot");

		Capability c1 = new Capability("toto#toto");
		Capability c2 = new Capability("toto#toto");
		Capability c3 = new Capability("titi#titi");

		c1.setCapabilityURI("blabla");
		c2.setCapabilityURI("blabla");
		c3.setCapabilityURI("blibli");

		s1.addCapability(c1);
		s2.addCapability(c1);
		s3.addCapability(c2);
		s4.addCapability(c3);
		s5.addCapability(c1);
		s5.addCapability(c3);

		// true
		assertTrue("same capa", s1.equals(s2));
		assertTrue("capa same semantic", s1.equals(s3));
		assertTrue("same objetc", s1.equals(s1));
		assertTrue("same capa 2", s2.equals(s1));
		assertTrue("capa same semantic 2", s3.equals(s1));

		// false
		assertFalse("not same Capa", s1.equals(s4));
		assertFalse("not same Capa 2", s4.equals(s1));
		assertFalse("not same Capa number", s1.equals(s5));
		assertFalse("not same Capa number 2", s5.equals(s1));

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#equals(java.lang.Object)}.
	 * <br>
	 * (add required capability)
	 */
	@Test
	public final void testEqualsObjectWithRequiredCapability() {

		/*
		 * Test Required Capability
		 */
		Service s1 = new Service("toot");
		Service s2 = new Service("toot");
		Service s3 = new Service("toot");
		Service s4 = new Service("toot");
		Service s5 = new Service("toot");

		Capability c1 = new Capability("toto#toto");
		Capability c2 = new Capability("toto#toto");
		Capability c3 = new Capability("titi#titi");

		c1.setCapabilityURI("blabla");
		c2.setCapabilityURI("blabla");
		c3.setCapabilityURI("blibli");

		s1.addRequiredCapability(c1);
		s2.addRequiredCapability(c1);
		s3.addRequiredCapability(c2);
		s4.addRequiredCapability(c3);
		s5.addRequiredCapability(c1);
		s5.addRequiredCapability(c3);

		// true
		assertTrue("same capa", s1.equals(s2));
		assertTrue("capa same semantic", s1.equals(s3));
		assertTrue("same objetc", s1.equals(s1));
		assertTrue("same capa 2", s2.equals(s1));
		assertTrue("capa same semantic 2", s3.equals(s1));

		// false
		assertFalse("not same Capa", s1.equals(s4));
		assertFalse("not same Capa 2", s4.equals(s1));
		assertFalse("not same Capa number", s1.equals(s5));
		assertFalse("not same Capa number 2", s5.equals(s1));

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#equals(java.lang.Object)}.
	 * <br>
	 * (add provided capability)
	 */
	@Test
	public final void testEqualsObjectWithProvidedCapability() {

		/*
		 * Test Capability
		 */
		Service s1 = new Service("toot");
		Service s2 = new Service("toot");
		Service s3 = new Service("toot");
		Service s4 = new Service("toot");
		Service s5 = new Service("toot");

		Capability c1 = new Capability("toto#toto");
		Capability c2 = new Capability("toto#toto");
		Capability c3 = new Capability("titi#titi");

		c1.setCapabilityURI("blabla");
		c2.setCapabilityURI("blabla");
		c3.setCapabilityURI("blibli");

		s1.addProvidedCapability(c1);
		s2.addProvidedCapability(c1);
		s3.addProvidedCapability(c2);
		s4.addProvidedCapability(c3);
		s5.addProvidedCapability(c1);
		s5.addProvidedCapability(c3);

		// true
		assertTrue("same capa", s1.equals(s2));
		assertTrue("capa same semantic", s1.equals(s3));
		assertTrue("same objetc", s1.equals(s1));
		assertTrue("same capa 2", s2.equals(s1));
		assertTrue("capa same semantic 2", s3.equals(s1));

		// false
		assertFalse("not same Capa", s1.equals(s4));
		assertFalse("not same Capa 2", s4.equals(s1));
		assertFalse("not same Capa number", s1.equals(s5));
		assertFalse("not same Capa number 2", s5.equals(s1));

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#containsTheCapability(fr.inria.icocoa.servicemodel.Capability)}.
	 */
	@Test
	public final void testContainsTheCapability() {
		Service s1 = new Service();
		Service s2 = new Service();
		Service s3 = new Service();
		Service s4 = new Service();

		Capability c1 = new Capability("toto#toto");

		s2.addCapability(c1);
		s3.addProvidedCapability(c1);
		s4.addRequiredCapability(c1);

		int originalType = c1.getTypeOfCapability();

		// not contained
		assertFalse("not contains with same object", s1
				.containsTheCapability(c1));
		assertTrue("type restored 1", c1.getTypeOfCapability() == originalType);
		assertFalse("not contains with equivalent object", s1
				.containsTheCapability(new Capability("toto#toto")));

		// contained
		assertTrue("contains with same object (SC)", s2
				.containsTheCapability(c1));
		assertTrue("type restored 2", c1.getTypeOfCapability() == originalType);
		assertTrue("contains with equivalent object (SC)", s2
				.containsTheCapability(new Capability("toto#toto")));

		assertTrue("contains with same object (R)", s4
				.containsTheCapability(c1));
		assertTrue("type restored 3", c1.getTypeOfCapability() == originalType);
		assertTrue("contains with equivalent object (R)", s4
				.containsTheCapability(new Capability("toto#toto")));

		assertTrue("contains with same object (P)", s3
				.containsTheCapability(c1));
		assertTrue("type restored 4", c1.getTypeOfCapability() == originalType);
		assertTrue("contains with equivalent object (P)", s3
				.containsTheCapability(new Capability("toto#toto")));

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#addRequiredCapability_E(fr.inria.icocoa.servicemodel.Capability)}.
	 * (add same object)
	 * 
	 * @throws CapabilityAlreadyRegisteredException
	 */
	@Test(expected = fr.inria.icocoa.exception.CapabilityAlreadyRegisteredException.class)
	public final void testAddRequiredCapability_E1()
			throws CapabilityAlreadyRegisteredException {
		Service s = new Service("toto");
		Capability c = new Capability("a#a", "b");
		Capability c2 = new Capability("c#c", "d");
		c.setCapabilityURI("blabla");
		c2.setCapabilityURI("blibla");
		try {
			s.addRequiredCapability_E(c);
		} catch (CapabilityAlreadyRegisteredException e) {
			// this is the first add so no exception is expected
			fail("first add failed !!!");
		} catch (SameURIException e) {
			fail("same URI exception 1");
		}
		try {
			s.addRequiredCapability_E(c2);
		} catch (CapabilityAlreadyRegisteredException e) {
			// this is the first add of this object so no exception is expected
			fail("second add failed !!!");
		} catch (SameURIException e) {
			fail("same URI exception 2");
		}
		try {
			s.addRequiredCapability_E(c);
		} catch (CapabilityAlreadyRegisteredException e) {
			// this time the exception is expected
			throw e;
		} catch (SameURIException e) {
			fail("same URI exception 2");
		}

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#addRequiredCapability_E(fr.inria.icocoa.servicemodel.Capability)}.
	 * (add of an equivalent object)
	 * 
	 * @throws CapabilityAlreadyRegisteredException
	 * 
	 */
	@Test(expected = fr.inria.icocoa.exception.CapabilityAlreadyRegisteredException.class)
	public final void testAddRequiredCapability_E2()
			throws CapabilityAlreadyRegisteredException {
		Service s = new Service("toto");
		Capability c = new Capability("a#a", "b");
		Capability c2 = new Capability("a#a", "b");
		c.setCapabilityURI("blabla");
		c2.setCapabilityURI("blabla");
		try {
			s.addRequiredCapability_E(c);
		} catch (CapabilityAlreadyRegisteredException e) {
			// this is the first add so no exception is expected
			fail("first add failed !!!");
		} catch (SameURIException e) {
			fail("same URI exception 1");
		}
		try {
			s.addRequiredCapability_E(c2);
		} catch (CapabilityAlreadyRegisteredException e) {
			// this time the exception is expected
			throw e;
		} catch (SameURIException e) {
			fail("same URI exception 2");
		}

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#addRequiredCapability_E(fr.inria.icocoa.servicemodel.Capability)}.
	 * (add two different object but with same CapaURI)
	 * 
	 * @throws SameURIException
	 * 
	 */
	@Test(expected = fr.inria.icocoa.exception.SameURIException.class)
	public final void testRequiredAddCapability_E3() throws SameURIException {
		Service s = new Service("toto");
		Capability c = new Capability("a#a", "b");
		Capability c2 = new Capability("e#e", "f");
		c.setCapabilityURI("blabla");
		c2.setCapabilityURI("blabla");
		try {
			s.addRequiredCapability_E(c);
		} catch (CapabilityAlreadyRegisteredException e) {
			// this is the first add so no exception is expected
			fail("first add failed !!!");
		} catch (SameURIException e) {
			fail("same URI exception not expected");
		}
		try {
			s.addRequiredCapability_E(c2);
		} catch (CapabilityAlreadyRegisteredException e) {
			fail("CapabilityAlreadyRegisteredException unexpected");
		} catch (SameURIException e) {
			// exception expected
			throw e;
		}

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#addProvidedCapability(fr.inria.icocoa.servicemodel.Capability)}.
	 * 
	 */
	@Test
	public final void testAddRequiredCapability() {
		Service s = new Service("toto");
		Capability c = new Capability("a#a", "b");
		Capability c2 = new Capability("c#c", "d");
		Capability c3 = new Capability("a#a", "b");
		Capability c4 = new Capability("e#e", "f");

		c.setCapabilityURI("blabla");
		c2.setCapabilityURI("blibli");
		c3.setCapabilityURI("blabla");
		c4.setCapabilityURI("bloblo");

		assertTrue("first addition", s.addRequiredCapability(c));
		assertTrue("verification 1 of nb of capabiliy (add ok)", s
				.getCapabilitiesMap().size() == 0);
		assertTrue("verification 1 bis of nb of capabiliy (add ok)", s
				.getRequired_Capabilities().size() == 1);
		assertTrue("verification 1 ter of nb of capabiliy (add ok)", s
				.getProvided_Capabilities().size() == 0);
		assertTrue("verification 1 of the change of type", c
				.getTypeOfCapability() == Capability.REQUIRED);
		assertTrue("verification 1 of list of keys", s
				.getRequired_Capabilities().containsKey("blabla"));

		assertTrue("second addition", s.addRequiredCapability(c2));
		assertTrue("verification 2 of nb of capabiliy (add ok)", s
				.getCapabilitiesMap().size() == 0);
		assertTrue("verification 2 bis of nb of capabiliy (add ok)", s
				.getRequired_Capabilities().size() == 2);
		assertTrue("verification 2 ter of nb of capabiliy (add ok)", s
				.getProvided_Capabilities().size() == 0);
		assertTrue("verification 2 of the change of type", c2
				.getTypeOfCapability() == Capability.REQUIRED);
		assertTrue("verification 2 of list of keys", s
				.getRequired_Capabilities().containsKey("blabla")
				&& s.getRequired_Capabilities().containsKey("blibli"));

		assertFalse("second addition of the same object", s
				.addRequiredCapability(c2));
		assertTrue("verification 3 of nb of capabiliy (add ok)", s
				.getCapabilitiesMap().size() == 0);
		assertTrue("verification 3 bis of nb of capabiliy (add ok)", s
				.getRequired_Capabilities().size() == 2);
		assertTrue("verification 3 ter of nb of capabiliy (add ok)", s
				.getProvided_Capabilities().size() == 0);
		assertTrue("verification 3 of the change of type", c2
				.getTypeOfCapability() == Capability.REQUIRED);

		assertFalse("second addition of an equivalent object", s
				.addRequiredCapability(c3));
		assertTrue("verification 4 of nb of capabiliy (add ok)", s
				.getCapabilitiesMap().size() == 0);
		assertTrue("verification 4 bis of nb of capabiliy (add ok)", s
				.getRequired_Capabilities().size() == 2);
		assertTrue("verification 4 ter of nb of capabiliy (add ok)", s
				.getProvided_Capabilities().size() == 0);
		assertTrue("verification 4 of the change of type", c2
				.getTypeOfCapability() == Capability.REQUIRED);
		assertTrue("verification 4 of the change of type", c
				.getTypeOfCapability() == Capability.REQUIRED);

		assertFalse("add an different object with the same CapabilityURI", s
				.addRequiredCapability(c3));
		assertTrue("verification 4 of nb of capabiliy (add ok)", s
				.getCapabilitiesMap().size() == 0);
		assertTrue("verification 4 bis of nb of capabiliy (add ok)", s
				.getRequired_Capabilities().size() == 2);
		assertTrue("verification 4 ter of nb of capabiliy (add ok)", s
				.getProvided_Capabilities().size() == 0);
		assertTrue("verification 4 of the change of type", c2
				.getTypeOfCapability() == Capability.REQUIRED);
		assertTrue("verification 4 of the change of type", c
				.getTypeOfCapability() == Capability.REQUIRED);

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#addProvidedCapability_E(fr.inria.icocoa.servicemodel.Capability)}.
	 * (add same object)
	 * 
	 * @throws CapabilityAlreadyRegisteredException
	 */
	@Test(expected = fr.inria.icocoa.exception.CapabilityAlreadyRegisteredException.class)
	public final void testAddProvidedCapability_E1()
			throws CapabilityAlreadyRegisteredException {
		Service s = new Service("toto");
		Capability c = new Capability("a#a", "b");
		Capability c2 = new Capability("c#c", "d");
		c.setCapabilityURI("blabla");
		c2.setCapabilityURI("blibla");
		try {
			s.addProvidedCapability_E(c);
		} catch (CapabilityAlreadyRegisteredException e) {
			// this is the first add so no exception is expected
			fail("first add failed !!!");
		} catch (SameURIException e) {
			fail("same URI exception 1");
		}
		try {
			s.addProvidedCapability_E(c2);
		} catch (CapabilityAlreadyRegisteredException e) {
			// this is the first add of this object so no exception is expected
			fail("second add failed !!!");
		} catch (SameURIException e) {
			fail("same URI exception 2");
		}
		try {
			s.addProvidedCapability_E(c);
		} catch (CapabilityAlreadyRegisteredException e) {
			// this time the exception is expected
			throw e;
		} catch (SameURIException e) {
			fail("same URI exception 2");
		}

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#addProvidedCapability_E(fr.inria.icocoa.servicemodel.Capability)}.
	 * (add of an equivalent object)
	 * 
	 * @throws CapabilityAlreadyRegisteredException
	 * 
	 */
	@Test(expected = fr.inria.icocoa.exception.CapabilityAlreadyRegisteredException.class)
	public final void testAddProvidedCapability_E2()
			throws CapabilityAlreadyRegisteredException {
		Service s = new Service("toto");
		Capability c = new Capability("a#a", "b");
		Capability c2 = new Capability("a#a", "b");
		c.setCapabilityURI("blabla");
		c2.setCapabilityURI("blabla");
		try {
			s.addProvidedCapability_E(c);
		} catch (CapabilityAlreadyRegisteredException e) {
			// this is the first add so no exception is expected
			fail("first add failed !!!");
		} catch (SameURIException e) {
			fail("same URI exception 1");
		}
		try {
			s.addProvidedCapability_E(c2);
		} catch (CapabilityAlreadyRegisteredException e) {
			// this time the exception is expected
			throw e;
		} catch (SameURIException e) {
			fail("same URI exception 2");
		}

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#addProvidedCapability_E(fr.inria.icocoa.servicemodel.Capability)}.
	 * (add two different object but with same CapaURI)
	 * 
	 * @throws SameURIException
	 * 
	 */
	@Test(expected = fr.inria.icocoa.exception.SameURIException.class)
	public final void testProvidedAddCapability_E3() throws SameURIException {
		Service s = new Service("toto");
		Capability c = new Capability("a#a", "b");
		Capability c2 = new Capability("e#e", "f");
		c.setCapabilityURI("blabla");
		c2.setCapabilityURI("blabla");
		try {
			s.addProvidedCapability_E(c);
		} catch (CapabilityAlreadyRegisteredException e) {
			// this is the first add so no exception is expected
			fail("first add failed !!!");
		} catch (SameURIException e) {
			fail("same URI exception not expected");
		}
		try {
			s.addProvidedCapability_E(c2);
		} catch (CapabilityAlreadyRegisteredException e) {
			fail("CapabilityAlreadyRegisteredException unexpected");
		} catch (SameURIException e) {
			// exception expected
			throw e;
		}

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#addProvidedCapability(fr.inria.icocoa.servicemodel.Capability)}.
	 * 
	 */
	@Test
	public final void testAddProvidedCapability() {
		Service s = new Service("toto");
		Capability c = new Capability("a#a", "b");
		Capability c2 = new Capability("c#c", "d");
		Capability c3 = new Capability("a#a", "b");
		Capability c4 = new Capability("e#e", "f");

		c.setCapabilityURI("blabla");
		c2.setCapabilityURI("blibli");
		c3.setCapabilityURI("blabla");
		c4.setCapabilityURI("bloblo");

		assertTrue("first addition", s.addProvidedCapability(c));
		assertTrue("verification 1 of nb of capabiliy (add ok)", s
				.getCapabilitiesMap().size() == 0);
		assertTrue("verification 1 bis of nb of capabiliy (add ok)", s
				.getRequired_Capabilities().size() == 0);
		assertTrue("verification 1 ter of nb of capabiliy (add ok)", s
				.getProvided_Capabilities().size() == 1);
		assertTrue("verification 1 of the change of type", c
				.getTypeOfCapability() == Capability.PROVIDED);
		assertTrue("verification 1 of list of keys", s
				.getProvided_Capabilities().containsKey("blabla"));

		assertTrue("second addition", s.addProvidedCapability(c2));
		assertTrue("verification 2 of nb of capabiliy (add ok)", s
				.getCapabilitiesMap().size() == 0);
		assertTrue("verification 2 bis of nb of capabiliy (add ok)", s
				.getRequired_Capabilities().size() == 0);
		assertTrue("verification 2 ter of nb of capabiliy (add ok)", s
				.getProvided_Capabilities().size() == 2);
		assertTrue("verification 2 of the change of type", c2
				.getTypeOfCapability() == Capability.PROVIDED);
		assertTrue("verification 2 of list of keys", s
				.getProvided_Capabilities().containsKey("blabla")
				&& s.getProvided_Capabilities().containsKey("blibli"));

		assertFalse("second addition of the same object", s
				.addProvidedCapability(c2));
		assertTrue("verification 3 of nb of capabiliy (add ok)", s
				.getCapabilitiesMap().size() == 0);
		assertTrue("verification 3 bis of nb of capabiliy (add ok)", s
				.getRequired_Capabilities().size() == 0);
		assertTrue("verification 3 ter of nb of capabiliy (add ok)", s
				.getProvided_Capabilities().size() == 2);
		assertTrue("verification 3 of the change of type", c2
				.getTypeOfCapability() == Capability.PROVIDED);

		assertFalse("second addition of an equivalent object", s
				.addProvidedCapability(c3));
		assertTrue("verification 4 of nb of capabiliy (add ok)", s
				.getCapabilitiesMap().size() == 0);
		assertTrue("verification 4 bis of nb of capabiliy (add ok)", s
				.getRequired_Capabilities().size() == 0);
		assertTrue("verification 4 ter of nb of capabiliy (add ok)", s
				.getProvided_Capabilities().size() == 2);
		assertTrue("verification 4 of the change of type", c2
				.getTypeOfCapability() == Capability.PROVIDED);
		assertTrue("verification 4 of the change of type", c
				.getTypeOfCapability() == Capability.PROVIDED);

		assertFalse("add an different object with the same CapabilityURI", s
				.addProvidedCapability(c3));
		assertTrue("verification 4 of nb of capabiliy (add ok)", s
				.getCapabilitiesMap().size() == 0);
		assertTrue("verification 4 bis of nb of capabiliy (add ok)", s
				.getRequired_Capabilities().size() == 0);
		assertTrue("verification 4 ter of nb of capabiliy (add ok)", s
				.getProvided_Capabilities().size() == 2);
		assertTrue("verification 4 of the change of type", c2
				.getTypeOfCapability() == Capability.PROVIDED);
		assertTrue("verification 4 of the change of type", c
				.getTypeOfCapability() == Capability.PROVIDED);

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#addCapability_E(fr.inria.icocoa.servicemodel.Capability)}.
	 * (add same object)
	 * 
	 * @throws CapabilityAlreadyRegisteredException
	 */
	@Test(expected = fr.inria.icocoa.exception.CapabilityAlreadyRegisteredException.class)
	public final void testAddCapability_E1()
			throws CapabilityAlreadyRegisteredException {
		Service s = new Service("toto");
		Capability c = new Capability("a#a", "b");
		Capability c2 = new Capability("c#c", "d");
		c.setCapabilityURI("blabla");
		c2.setCapabilityURI("blibla");
		try {
			s.addCapability_E(c);
		} catch (CapabilityAlreadyRegisteredException e) {
			// this is the first add so no exception is expected
			fail("first add failed !!!");
		} catch (SameURIException e) {
			fail("same URI exception 1");
		}
		try {
			s.addCapability_E(c2);
		} catch (CapabilityAlreadyRegisteredException e) {
			// this is the first add of this object so no exception is expected
			fail("second add failed !!!");
		} catch (SameURIException e) {
			fail("same URI exception 2");
		}
		try {
			s.addCapability_E(c);
		} catch (CapabilityAlreadyRegisteredException e) {
			// this time the exception is expected
			throw e;
		} catch (SameURIException e) {
			fail("same URI exception 2");
		}

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#addCapability_E(fr.inria.icocoa.servicemodel.Capability)}.
	 * (add of an equivalent object)
	 * 
	 * @throws CapabilityAlreadyRegisteredException
	 * 
	 */
	@Test(expected = fr.inria.icocoa.exception.CapabilityAlreadyRegisteredException.class)
	public final void testAddCapability_E2()
			throws CapabilityAlreadyRegisteredException {
		Service s = new Service("toto");
		Capability c = new Capability("a#a", "b");
		Capability c2 = new Capability("a#a", "b");
		c.setCapabilityURI("blabla");
		c2.setCapabilityURI("blabla");
		try {
			s.addCapability_E(c);
		} catch (CapabilityAlreadyRegisteredException e) {
			// this is the first add so no exception is expected
			fail("first add failed !!!");
		} catch (SameURIException e) {
			fail("same URI exception 1");
		}
		try {
			s.addCapability_E(c2);
		} catch (CapabilityAlreadyRegisteredException e) {
			// this time the exception is expected
			throw e;
		} catch (SameURIException e) {
			fail("same URI exception 2");
		}

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#addCapability_E(fr.inria.icocoa.servicemodel.Capability)}.
	 * (add two different object but with same CapaURI)
	 * 
	 * @throws SameURIException
	 * 
	 */
	@Test(expected = fr.inria.icocoa.exception.SameURIException.class)
	public final void testAddCapability_E3() throws SameURIException {
		Service s = new Service("toto");
		Capability c = new Capability("a#a", "b");
		Capability c2 = new Capability("e#e", "f");
		c.setCapabilityURI("blabla");
		c2.setCapabilityURI("blabla");
		try {
			s.addCapability_E(c);
		} catch (CapabilityAlreadyRegisteredException e) {
			// this is the first add so no exception is expected
			fail("first add failed !!!");
		} catch (SameURIException e) {
			fail("same URI exception not expetcetd");
		}
		try {
			s.addCapability_E(c2);
		} catch (CapabilityAlreadyRegisteredException e) {
			fail("CapabilityAlreadyRegisteredException unexpetced");
		} catch (SameURIException e) {
			// exception expected
			throw e;
		}

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#addCapability(fr.inria.icocoa.servicemodel.Capability)}.
	 * 
	 */
	@Test
	public final void testAddCapability() {
		Service s = new Service("toto");
		Capability c = new Capability("a#a", "b");
		Capability c2 = new Capability("c#c", "d");
		Capability c3 = new Capability("a#a", "b");
		Capability c4 = new Capability("e#e", "f");

		c.setCapabilityURI("blabla");
		c2.setCapabilityURI("blibli");
		c3.setCapabilityURI("blabla");
		c4.setCapabilityURI("bloblo");

		assertTrue("first addition", s.addCapability(c));
		assertTrue("verification 1 of nb of capabiliy (add ok)", s
				.getCapabilitiesMap().size() == 1);
		assertTrue("verification 1 bis of nb of capabiliy (add ok)", s
				.getRequired_Capabilities().size() == 0);
		assertTrue("verification 1 ter of nb of capabiliy (add ok)", s
				.getProvided_Capabilities().size() == 0);
		assertTrue("verification 1 of the change of type", c
				.getTypeOfCapability() == Capability.SUBCAPABILITY);
		assertTrue("verification 1 of list of keys", s.getCapabilitiesMap()
				.containsKey("blabla"));

		assertTrue("second addition", s.addCapability(c2));
		assertTrue("verification 2 of nb of capabiliy (add ok)", s
				.getCapabilitiesMap().size() == 2);
		assertTrue("verification 2 bis of nb of capabiliy (add ok)", s
				.getRequired_Capabilities().size() == 0);
		assertTrue("verification 2 ter of nb of capabiliy (add ok)", s
				.getProvided_Capabilities().size() == 0);
		assertTrue("verification 2 of the change of type", c2
				.getTypeOfCapability() == Capability.SUBCAPABILITY);
		assertTrue("verification 2 of list of keys", s.getCapabilitiesMap()
				.containsKey("blabla")
				&& s.getCapabilitiesMap().containsKey("blibli"));

		assertFalse("second addition of the same object", s.addCapability(c2));
		assertTrue("verification 3 of nb of capabiliy (add ok)", s
				.getCapabilitiesMap().size() == 2);
		assertTrue("verification 3 bis of nb of capabiliy (add ok)", s
				.getRequired_Capabilities().size() == 0);
		assertTrue("verification 3 ter of nb of capabiliy (add ok)", s
				.getProvided_Capabilities().size() == 0);
		assertTrue("verification 3 of the change of type", c2
				.getTypeOfCapability() == Capability.SUBCAPABILITY);

		assertFalse("second addition of an equivalent object", s
				.addCapability(c3));
		assertTrue("verification 4 of nb of capabiliy (add ok)", s
				.getCapabilitiesMap().size() == 2);
		assertTrue("verification 4 bis of nb of capabiliy (add ok)", s
				.getRequired_Capabilities().size() == 0);
		assertTrue("verification 4 ter of nb of capabiliy (add ok)", s
				.getProvided_Capabilities().size() == 0);
		assertTrue("verification 4 of the change of type", c2
				.getTypeOfCapability() == Capability.SUBCAPABILITY);
		assertTrue("verification 4 of the change of type", c
				.getTypeOfCapability() == Capability.SUBCAPABILITY);

		assertFalse("add an different object with the same CapabilityURI", s
				.addCapability(c3));
		assertTrue("verification 4 of nb of capabiliy (add ok)", s
				.getCapabilitiesMap().size() == 2);
		assertTrue("verification 4 bis of nb of capabiliy (add ok)", s
				.getRequired_Capabilities().size() == 0);
		assertTrue("verification 4 ter of nb of capabiliy (add ok)", s
				.getProvided_Capabilities().size() == 0);
		assertTrue("verification 4 of the change of type", c2
				.getTypeOfCapability() == Capability.SUBCAPABILITY);
		assertTrue("verification 4 of the change of type", c
				.getTypeOfCapability() == Capability.SUBCAPABILITY);

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#findInputInService(java.lang.String)}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testFindInputInService() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#findOutputInService(java.lang.String)}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testFindOutputInService() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#findCapabilityInService(java.lang.String)}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testFindCapabilityInService() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#getInputs(java.lang.String)}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testGetInputs() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#getOutputs(java.lang.String)}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testGetOutputs() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#getRequiredTask()}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testGetRequiredTask() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#getTaskProcess(java.lang.String)}.
	 */
	@Test
	public final void testGetTaskProcess() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#isAtomic(java.lang.String)}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testIsAtomic() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#isComposite(java.lang.String)}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testIsComposite() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#getRequiredCapability(java.lang.String)}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testGetRequiredCapability() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#setProvidedCapabilities(java.util.List)}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testSetProvidedCapabilities() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#getProvidedCapabilities()}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testGetProvidedCapabilities() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#setRequiredCapabilities(java.util.List)}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testSetRequiredCapabilities() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#getRequiredCapabilities()}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testGetRequiredCapabilities() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#setCapabilities(java.util.List)}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testSetCapabilitiesListOfCapability() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#getCapabilities()}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testGetCapabilities() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#setCapabilities(java.util.Map)}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testSetCapabilitiesMapOfStringCapability() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#getProvidedCapability(java.lang.String)}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testGetProvidedCapability() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#getCapability(java.lang.String)}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testGetCapability() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#getCapabilityFromTypeName(java.lang.String)}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testGetCapabilityFromName() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#getCapabilityFromURI(java.lang.String)}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testGetCapabilityFromURI() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#findFirstCapabilityUsing(fr.inria.icocoa.servicemodel.IO, java.lang.String)}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testFindFirstCapabilityUsing() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#findLastCapabilityUsing(fr.inria.icocoa.servicemodel.IO, java.lang.String)}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testFindLastCapabilityUsing() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#getCapabilityFromConversationName(java.lang.String)}.
	 */
	@Test
	public final void testGetCapabilityFromConversationName() {
		// TODO test to implements
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#getCapabilityFromTypeName(java.lang.String)}.
	 * 
	 * @todo test to implements
	 */
	@Test
	public final void testGetCapabilityFromTypeName() {
		Capability capa1 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa1");
		Capability capa2 = new Capability(
				"http:/www.inria.fr/myPiOntology.owl#SemCapa2", null, "capa2");
		Capability capa3 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa3");
		Capability capa4 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa2", null, "capa4");
		Capability capa5 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa5");
		Capability capa6 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa2", null, "capa6");

		Service s1 = new Service("Tototitoa");
		s1.setType("hye#ho");
		s1.addProvidedCapability(capa1);
		s1.addProvidedCapability(capa2);
		s1.addCapability(capa3);
		s1.addCapability(capa4);
		s1.addRequiredCapability(capa5);
		s1.addRequiredCapability(capa6);

		List<Capability> l1 = s1.getCapabilityFromTypeName("SemCapa1");

		List<Capability> expected = new ArrayList<Capability>();
		expected.add(capa1);
		expected.add(capa3);

		// DebugPrinter.print(l1,"list result", 1);
		assertTrue("get Capability from type", Tools.compareList(l1, expected));

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.Service#partialExactMatch(fr.inria.icocoa.servicemodel.UserTask)}. *
	 * 
	 * @throws SemanticTypeNotKnowException
	 */
	@Test
	public final void testPartialExactMatch()
			throws SemanticTypeNotKnowException {
		// Reference Capa
		Input in1 = new Input("in1",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in2 = new Input("in2",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in3 = new Input("in3",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out1 = new Output("out1",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out2 = new Output("out2",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out3 = new Output("out3",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa1 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa1");

		capa1.addInput(in1);
		capa1.addInput(in2);
		capa1.addInput(in3);

		capa1.addOutput(out1);
		capa1.addOutput(out2);
		capa1.addOutput(out3);

		// Reference Capa 2
		Input in11 = new Input("in11",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in21 = new Input("in21",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in31 = new Input("in31",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out11 = new Output("out11",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out21 = new Output("out21",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out31 = new Output("out31",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa11 = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa2", null, "capa11");

		capa11.addInput(in11);
		capa11.addInput(in21);
		capa11.addInput(in31);

		capa11.addOutput(out11);
		capa11.addOutput(out21);
		capa11.addOutput(out31);

		Service s1 = new Service("Tototitoa");
		s1.setType("hye#ho");
		s1.addProvidedCapability(capa1);
		s1.addCapability(capa11);

		// Other Capa
		Input in1a = new Input("in1a",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in2a = new Input("in2a",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in3a = new Input("in3a",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out1a = new Output("out1a",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out2a = new Output("out2a",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out3a = new Output("out3a",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa1a = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa1", null, "capa1a");

		capa1a.addInput(in1a);
		capa1a.addInput(in2a);
		capa1a.addInput(in3a);

		capa1a.addOutput(out1a);
		capa1a.addOutput(out2a);
		capa1a.addOutput(out3a);

		// Other Capa
		Input in11a = new Input("in11a",
				"http://www.inria.fr/myPiOntology.owl#SemIn1");
		Input in21a = new Input("in21a",
				"http://www.inria.fr/myPiOntology.owl#SemIn2");
		Input in31a = new Input("in31a",
				"http://www.inria.fr/myPiOntology.owl#SemIn3");

		Output out11a = new Output("out11a",
				"http://www.inria.fr/myPiOntology.owl#SemOut1");
		Output out21a = new Output("out21a",
				"http://www.inria.fr/myPiOntology.owl#SemOut2");
		Output out31a = new Output("out31a",
				"http://www.inria.fr/myPiOntology.owl#SemOut3");

		Capability capa11a = new Capability(
				"http://www.inria.fr/myPiOntology.owl#SemCapa2", null,
				"capa11a");

		capa11a.addInput(in11a);
		capa11a.addInput(in21a);
		capa11a.addInput(in31a);

		capa11a.addOutput(out11a);
		capa11a.addOutput(out21a);
		capa11a.addOutput(out31a);

		UserTask u1 = new UserTask(new Service("Usertask titoa"));
		u1.addRequiredCapability(capa1a);
		u1.addCapability(capa11a);

		Matcher matcher = new SyntacticMatcher();
		ServiceMatch sm = matcher.serviceMatch(s1, u1);

		assertNotNull("test match capa 1", capa1.match(capa1a, matcher));
		assertNotNull("test match capa 2", capa11.match(capa11a, matcher));

		// DebugPrinter.print("sm =\n"+sm,1);

		// test not null
		assertNotNull("test not null", sm);

		CapabilityMatch expected = new CapabilityMatch(capa1a,capa1, MatchingType.EXACT);

		expected.addIOMatching(new IOMatch(in1a,in1,MatchingType.EXACT));
		expected.addIOMatching(new IOMatch(in2a,in2,MatchingType.EXACT));
		expected.addIOMatching(new IOMatch(in3a,in3,MatchingType.EXACT));
		
		expected.addIOMatching(new IOMatch(out1a,out1,MatchingType.EXACT));
		expected.addIOMatching(new IOMatch(out2a,out2,MatchingType.EXACT));
		expected.addIOMatching(new IOMatch(out3a,out3,MatchingType.EXACT));

		CapabilityMatch expected2 = new CapabilityMatch(capa11a,capa11, MatchingType.EXACT);
		
		expected2.addIOMatching(new IOMatch(in11a,in11,MatchingType.EXACT));
		expected2.addIOMatching(new IOMatch(in21a,in21,MatchingType.EXACT));
		expected2.addIOMatching(new IOMatch(in31a,in31,MatchingType.EXACT));
		
		expected2.addIOMatching(new IOMatch(out11a,out11,MatchingType.EXACT));
		expected2.addIOMatching(new IOMatch(out21a,out21,MatchingType.EXACT));
		expected2.addIOMatching(new IOMatch(out31a,out31,MatchingType.EXACT));

		ServiceMatch expected_s = new ServiceMatch(u1, s1, MatchingType.EXACT);
		expected_s.addCapabilityMatch(expected);
		expected_s.addCapabilityMatch(expected2);

		// test equality
		assertEquals("CapabilityMatch 1", sm.getCapabilityMatches().get(0),
				expected);
		assertEquals("CapabilityMatch 2", sm.getCapabilityMatches().get(1),
				expected2);
		assertEquals("Test equality", sm, expected_s);

	}

}
