/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.junittest.servicemodel.domparser;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.HashMap;
import java.util.Map;

import org.junit.Test;

import fr.inria.icocoa.servicemodel.Input;
import fr.inria.icocoa.servicemodel.Transferable;
import fr.inria.icocoa.servicemodel.domparser.toolobjects.ConcurrentDataMap;

/**
 * ConcurrentDataMapTest.java
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Jul 31, 2007 10:28:25 AM
 */
public class ConcurrentDataMapTest {

	/**
	 * 
	 * ConcurrentDataMapTest.java<br>
	 * 
	 * Thread that add a list of value (given as parameter) in a
	 * ConcurrentDataMap (also given as parameter)
	 * 
	 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> -
	 *         <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
	 * 
	 * Creation Date : Jul 31, 2007 10:39:37 AM
	 * @param <K>
	 * @param <V>
	 */
	private class MyThread<K, V extends Transferable<V>> extends Thread {

		/**
		 * Reference to the map to fill in
		 */
		private ConcurrentDataMap<K, V> map;

		/**
		 * Map of value to add in the map
		 */
		private Map<K, V> valueToAdd;

		/**
		 * Create the Thread by giving the map to fill in and the map of value
		 * to add
		 * 
		 * @param theMap
		 *            the ConcurrentDataMap to fill in
		 * @param toAdd
		 *            the map of value to add in the DataMap
		 */
		public MyThread(ConcurrentDataMap<K, V> theMap, Map<K, V> toAdd) {
			this.map = theMap;
			this.valueToAdd = toAdd;
		}

		/**
		 * Walk through the map of value to add and insert it into the
		 * ConcurrentDataMap.
		 */
		public void run() {
			for (K k : this.valueToAdd.keySet()) {
				map.addElement(k, this.valueToAdd.get(k));
			}
		}
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.domparser.toolobjects.ConcurrentDataMap#addElement(java.lang.Object, fr.inria.icocoa.servicemodel.Transferable)}.
	 */
	@Test
	public final void testAddElement() {
		ConcurrentDataMap<String, Input> map = new ConcurrentDataMap<String, Input>();

		Input a = new Input("a", "a#a");
		Input b = new Input("b", "b#b");
		Input c = new Input("c", "c#c");
		Input d = new Input("d", "d#d");
		Input e = new Input("e", "e#e");
		Input f = new Input("f", "f#f");
		Input g = new Input("g", "g#g");
		Input h = new Input("h", "h#h");
		Input i = new Input("i", "i#i");
		Input j = new Input("j", "j#j");
		Input k = new Input("k", "k#k");
		Input l = new Input("l", "l#l");

		/*
		 * Concurrent add of distinct elements
		 */
		Map<String, Input> m1 = new HashMap<String, Input>();
		m1.put(a.getLocalName(), a);
		m1.put(b.getLocalName(), b);
		Map<String, Input> m2 = new HashMap<String, Input>();
		m2.put(c.getLocalName(), c);
		m2.put(d.getLocalName(), d);
		Map<String, Input> m3 = new HashMap<String, Input>();
		m3.put(e.getLocalName(), e);
		m3.put(f.getLocalName(), f);
		m3.put(g.getLocalName(), g);
		Map<String, Input> m4 = new HashMap<String, Input>();
		m4.put(h.getLocalName(), h);
		m4.put(i.getLocalName(), i);
		m4.put(j.getLocalName(), j);
		m4.put(k.getLocalName(), k);
		m4.put(l.getLocalName(), l);

		Thread[] t = new Thread[] { new MyThread<String, Input>(map, m1),
				new MyThread<String, Input>(map, m2),
				new MyThread<String, Input>(map, m3),
				new MyThread<String, Input>(map, m4) };

		for (int it = t.length - 1; it >= 0; it--) {
			t[it].run();
		}

		// DebugPrinter.print("map =\n" + map, DebugPrinter.INFO);

		assertEquals("test number of element", 12, map.size());
		assertTrue("test map contains a", map.containsElement(a));
		assertTrue("test map contains b", map.containsElement(b));
		assertTrue("test map contains c", map.containsElement(c));
		assertTrue("test map contains d", map.containsElement(d));
		assertTrue("test map contains e", map.containsElement(e));
		assertTrue("test map contains f", map.containsElement(f));
		assertTrue("test map contains g", map.containsElement(g));
		assertTrue("test map contains h", map.containsElement(h));
		assertTrue("test map contains i", map.containsElement(i));
		assertTrue("test map contains j", map.containsElement(j));
		assertTrue("test map contains k", map.containsElement(k));
		assertTrue("test map contains l", map.containsElement(l));

		/*
		 * Concurrent add of more complex element
		 */
		Input a1 = new Input("a");
		Input b1 = new Input("b");
		Input c1 = new Input("c");
		Input d1 = new Input("d");
		Input e1 = new Input("e");
		Input f1 = new Input("f");
		Input g1 = new Input("g");
		Input h1 = new Input("h");
		Input i1 = new Input("i");
		Input j1 = new Input("j");
		Input k1 = new Input("k");
		Input l1 = new Input("l");

		Map<String, Input> m12 = new HashMap<String, Input>();
		m12.put(a.getLocalName(), a1);
		m12.put(b.getLocalName(), b1);
		Map<String, Input> m22 = new HashMap<String, Input>();
		m22.put(c.getLocalName(), c1);
		m22.put(d.getLocalName(), d1);
		Map<String, Input> m32 = new HashMap<String, Input>();
		m32.put(e.getLocalName(), e1);
		m32.put(f.getLocalName(), f1);
		m32.put(g.getLocalName(), g1);
		Map<String, Input> m42 = new HashMap<String, Input>();
		m42.put(h.getLocalName(), h1);
		m42.put(i.getLocalName(), i1);
		m42.put(j.getLocalName(), j1);
		m42.put(k.getLocalName(), k1);
		m42.put(l.getLocalName(), l1);

		ConcurrentDataMap<String, Input> map2 = new ConcurrentDataMap<String, Input>();

		Thread[] t2 = new Thread[] { new MyThread<String, Input>(map2, m1),
				new MyThread<String, Input>(map2, m2),
				new MyThread<String, Input>(map2, m3),
				new MyThread<String, Input>(map2, m4),
				new MyThread<String, Input>(map2, m12),
				new MyThread<String, Input>(map2, m22),
				new MyThread<String, Input>(map2, m32),
				new MyThread<String, Input>(map2, m42) };

		for (int it = t2.length - 1; it >= 0; it--) {
			t2[it].run();
		}

		// DebugPrinter.print("map2 =\n" + map2, DebugPrinter.INFO);

		assertEquals("test2 number of element", 12, map.size());
		assertTrue("test map2 contains a", map2.containsElement(a));
		assertTrue("test map2 contains b", map2.containsElement(b));
		assertTrue("test map2 contains c", map2.containsElement(c));
		assertTrue("test map2 contains d", map2.containsElement(d));
		assertTrue("test map2 contains e", map2.containsElement(e));
		assertTrue("test map2 contains f", map2.containsElement(f));
		assertTrue("test map2 contains g", map2.containsElement(g));
		assertTrue("test map2 contains h", map2.containsElement(h));
		assertTrue("test map2 contains i", map2.containsElement(i));
		assertTrue("test map2 contains j", map2.containsElement(j));
		assertTrue("test map2 contains k", map2.containsElement(k));
		assertTrue("test map2 contains l", map2.containsElement(l));

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.domparser.toolobjects.ConcurrentDataMap#removeElement(java.lang.Object)}.
	 */
	@Test
	public final void testRemoveElement() {
		ConcurrentDataMap<String, Input> map = new ConcurrentDataMap<String, Input>();

		Input a = new Input("a", "a#a");
		Input b = new Input("b", "b#b");
		Input c = new Input("c", "c#c");
		Input d = new Input("d", "d#d");
		Input e = new Input("e", "e#e");
		Input f = new Input("f", "f#f");
		Input g = new Input("g", "g#g");
		Input h = new Input("h", "h#h");
		Input i = new Input("i", "i#i");
		Input j = new Input("j", "j#j");
		Input k = new Input("k", "k#k");
		Input l = new Input("l", "l#l");

		/*
		 * Concurrent add of distinct elements
		 */
		Map<String, Input> m1 = new HashMap<String, Input>();
		m1.put(a.getLocalName(), a);
		m1.put(b.getLocalName(), b);
		Map<String, Input> m2 = new HashMap<String, Input>();
		m2.put(c.getLocalName(), c);
		m2.put(d.getLocalName(), d);
		Map<String, Input> m3 = new HashMap<String, Input>();
		m3.put(e.getLocalName(), e);
		m3.put(f.getLocalName(), f);
		m3.put(g.getLocalName(), g);
		Map<String, Input> m4 = new HashMap<String, Input>();
		m4.put(h.getLocalName(), h);
		m4.put(i.getLocalName(), i);
		m4.put(j.getLocalName(), j);
		m4.put(k.getLocalName(), k);
		m4.put(l.getLocalName(), l);

		Thread[] t = new Thread[] { new MyThread<String, Input>(map, m1),
				new MyThread<String, Input>(map, m2),
				new MyThread<String, Input>(map, m3),
				new MyThread<String, Input>(map, m4) };

		for (int it = t.length - 1; it >= 0; it--) {
			t[it].run();
		}

		// DebugPrinter.print("map =\n" + map, DebugPrinter.INFO);

		assertEquals("test number of element", 12, map.size());
		assertTrue("test map contains a", map.containsElement(a));
		assertTrue("test map contains b", map.containsElement(b));
		assertTrue("test map contains c", map.containsElement(c));
		assertTrue("test map contains d", map.containsElement(d));
		assertTrue("test map contains e", map.containsElement(e));
		assertTrue("test map contains f", map.containsElement(f));
		assertTrue("test map contains g", map.containsElement(g));
		assertTrue("test map contains h", map.containsElement(h));
		assertTrue("test map contains i", map.containsElement(i));
		assertTrue("test map contains j", map.containsElement(j));
		assertTrue("test map contains k", map.containsElement(k));
		assertTrue("test map contains l", map.containsElement(l));

		/* remove an element */
		map.removeElement("a");
		assertEquals("test2 number of element", 11, map.size());
		assertFalse("test2 map not contains a", map.containsElement(a));
		assertTrue("test2 map contains b", map.containsElement(b));
		assertTrue("test2 map contains c", map.containsElement(c));
		assertTrue("test2 map contains d", map.containsElement(d));
		assertTrue("test2 map contains e", map.containsElement(e));
		assertTrue("test2 map contains f", map.containsElement(f));
		assertTrue("test2 map contains g", map.containsElement(g));
		assertTrue("test2 map contains h", map.containsElement(h));
		assertTrue("test2 map contains i", map.containsElement(i));
		assertTrue("test2 map contains j", map.containsElement(j));
		assertTrue("test2 map contains k", map.containsElement(k));
		assertTrue("test2 map contains l", map.containsElement(l));

	}

}
