/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.junittest.servicemodel.domparser;

import static org.junit.Assert.fail;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;

import org.junit.Test;
import org.xml.sax.SAXException;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.exception.ParserNotFoundException;
import fr.inria.icocoa.servicemodel.domparser.Parser;
import fr.inria.icocoa.servicemodel.domparser.ParserFactory;
import fr.inria.icocoa.servicemodel.domparser.specializedparsers.ConversationParserLink;
import fr.inria.icocoa.servicemodel.domparser.specializedparsers.InputInCapabilityParserLink;
import fr.inria.icocoa.servicemodel.domparser.specializedparsers.InputParserLink;
import fr.inria.icocoa.servicemodel.domparser.specializedparsers.OutputInCapabilityParserLink;
import fr.inria.icocoa.servicemodel.domparser.specializedparsers.OutputParserLink;
import fr.inria.icocoa.servicemodel.domparser.specializedparsers.QoSParserLink;

/**
 * ParserTest.java
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Aug 3, 2007 2:46:22 PM
 */
public class ParserTest {

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.domparser.Parser#parse(org.w3c.dom.Node)}.
	 */
	//@Test
	public final void testParseNode() {
		fail("Not yet implemented"); // TODO
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicemodel.domparser.Parser#parse(java.lang.String)}.
	 */
	@Test
	public final void testParseString() {
		String SDLfilename = System.getProperty("user.dir")
				+ "/amigo-s/ServiceB.owl";

		ParserFactory pf = ParserFactory.getInstanceOfFactory();

		try {
			pf.addParser(OutputParserLink.class.getName());
			pf.addParser(InputParserLink.class.getName());
			pf.addParser(OutputInCapabilityParserLink.class.getName());
			pf.addParser(InputInCapabilityParserLink.class.getName());
		} catch (ParserNotFoundException e1) {
			e1.printStackTrace();
		}

		Parser p = pf.getInstanceOfParser();

		DebugPrinter.print("ParserTest::testParseString() --> parser = \n"
				+ p.toString(), DebugPrinter.INFO);

		try {
			p.parse(SDLfilename);
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		DebugPrinter
				.print(
						"\n===========================================================================================",
						DebugPrinter.INFO);

		String SDLfilename2 = System.getProperty("user.dir")
				+ "/amigo-s/PiLowService.owl";

		ParserFactory.init();
		pf = ParserFactory.getInstanceOfFactory();

		try {
			pf.addParser(OutputParserLink.class.getName());
			pf.addParser(InputParserLink.class.getName());
			pf.addParser(OutputInCapabilityParserLink.class.getName());
			pf.addParser(InputInCapabilityParserLink.class.getName());
			pf.addParser(QoSParserLink.class.getName());
			pf.addParser(ConversationParserLink.class.getName());
		} catch (ParserNotFoundException e1) {
			e1.printStackTrace();
		}

		Parser p2 = pf.getInstanceOfParser();

		try {
			p2.parse(SDLfilename2);
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated
			e.printStackTrace();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		DebugPrinter
				.print(
						"\n===========================================================================================",
						DebugPrinter.INFO);

		/*
		 * String SDLfilename3 = System.getProperty("user.dir") +
		 * "/amigo-s/MultimediaBrowser.owl";
		 * 
		 * ParserFactory.init(); pf = ParserFactory.getInstance();
		 * 
		 * 
		 * pf.addParser(new OutputParserLink()); pf.addParser(new
		 * InputParserLink()); pf.addParser(new OutputInCapabilityParserLink());
		 * pf.addParser(new InputInCapabilityParserLink()); pf.addParser(new
		 * QoSParserLink()); pf.addParser(new ConversationParserLink());
		 * 
		 * 
		 * Parser p3 = pf.getInstanceOfParser();
		 * 
		 * 
		 * 
		 * try { p3.parse(SDLfilename3); } catch (ParserConfigurationException
		 * e) { // TODO Auto-generated catch block e.printStackTrace(); } catch
		 * (SAXException e) { // TODO Auto-generated catch block
		 * e.printStackTrace(); } catch (IOException e) { // TODO Auto-generated
		 * catch block e.printStackTrace(); } catch (InterruptedException e) { //
		 * TODO Auto-generated catch block e.printStackTrace(); }
		 */

	}

}
